<?php

/*
 * To authenticate Basic login credential
 */
require_once '..' . DIRECTORY_SEPARATOR . 'library' . DIRECTORY_SEPARATOR . 'bootstrap.php';
Bootstrap::initialize('static');
use Application\Session;
use Lazer\Classes\Database;
use Lazer\Classes\Helpers\Validate;
use Lazer\Classes\LazerException;
use Database\Connectors\ConnectionFactory;
use Application\Request;

header("Content-Type: application/json");

$postdata = file_get_contents("php://input");
$request  = json_decode($postdata, true);

$loggedInUser = null;
if (!empty($request)) {
    $isUserBlocked = isUserBlocked($request);
    if(!empty($isUserBlocked)) {
        echo json_encode(array(
            'success' => false,
            'field'   => 'username',
            'message' => 'User is blocked.',
        ));
        return;
    } else {
        $usersArray = getTheUserDetails($request['username']);
    }

    if(empty($usersArray)){
        logBadUsers($request);
        echo json_encode(array(
            'success' => false,
            'field'   => 'username',
            'message' => 'Username or Password is incorrect.',
        ));
        return;
    }
       

    $loggedInUser['username'] = $usersArray[0]['username'];
    $loggedInUser['userType'] = $usersArray[0]['user_type'];

    if (!empty($loggedInUser)) {

        if ($request['password'] === $usersArray[0]['password']) {

            Session::update(array(
                'username'  => $loggedInUser['username'],
                'userType'  => $loggedInUser['userType'],
                'access_urls' => array_slice($usersArray[0], 4),
                'loginFlag' => 'logged_in',
            ));
            echo json_encode(array(
                'success' => true,
                'message' => 'Correct Credential.',
            ));
        } else {
            logBadUsers($request);
            echo json_encode(array(
                'success' => false,
                'field'   => 'password',
                'message' => 'Incorrect Password',
            ));
        }
    } else {
        logBadUsers($request);
        echo json_encode(array(
            'success' => false,
            'field'   => 'username',
            'message' => 'Incorrect Username',
        ));
    }
} else {
    logBadUsers($request);
    echo json_encode(array(
        'success' => false,
        'message' => 'Incorrect Credential',
    ));
}


function getTheUserDetails($username)
{
    $table = array(
        'name' => 'users',
        'attr' => array(
            'id' => 'integer',
            'username' => 'string',
            'password'    => 'string',
            'email'    => 'string',
            'user_type' => 'integer',
            'change_access_permissions' => 'boolean',
            'ecommerce' => 'boolean',
            'campaigns' => 'boolean',
            'funnel_configurations' => 'boolean',
            'coupons' => 'boolean',
            'cms' => 'boolean',
            'extensions' => 'boolean',
            'logs' => 'boolean',
            'systems_log' => 'boolean',
            'user_activity' => 'boolean',
            'change_log' => 'boolean',
            'system' => 'boolean',
            'crm' => 'boolean',
            'users' => 'boolean',
            'settings' => 'boolean',
            'advance_settings' => 'boolean',
            'tools' => 'boolean',
            'affiliate_manager' => 'boolean',
            'pixel_manager' => 'boolean',
            'rotators' => 'boolean',
            'mid_routing' => 'boolean',
            'traffic_monitor' => 'boolean',
            'auto_responder' => 'boolean',
            'auto_filters' => 'boolean',
            'scheduler' => 'boolean',
            'troubleshooting' => 'boolean'
        ),
    );

    try {
        $user = Database::table('users')->where('username', '=', trim($username))->find()->asArray();
        return $user;
    }
    catch(Exception $e){
        return false;
    }
}

function logBadUsers($request) 
{
    try {
        $isTableCreated = createTableIfNotExists();
        
        if(empty($isTableCreated)) {
            return;
        }
        
        $dbConnection = getDatabaseConnection();
        $isUserExist = $dbConnection->table('blockUsers')
                            ->select('*')->where('user_name', $request['username'])->first();
        
        if(empty($isUserExist)) {
            $dateTime = new DateTime();
            $insertData = array(
                'user_name' => $request['username'],
                'ip_address' => Request::getClientIp(),
                'block_count' => 1,
                'created_at' => $dateTime->format('Y-m-d H:i:s')
            );

            $delayTime = 525600;
            $dateTime->modify(sprintf('+%d minute', $delayTime));
            $insertData['unblocked_at'] = $dateTime->format('Y-m-d H:i:s');
            $dbConnection->table('blockUsers')->insert($insertData);
        } else {
            $blockCount = $isUserExist['block_count'] + 1;
            $updateArray = array(
                'block_count' => $blockCount,
                'ip_address' => Request::getClientIp(),
            );
            $dbConnection->table('blockUsers')
                            ->where('id', $isUserExist['id'])
                            ->update($updateArray);
        }
        
    } catch (Exception $ex) {
    
    }
}

function isUserBlocked($request)
{
    try {
        $dateTime = new DateTime();
        $dbConnection = getDatabaseConnection();
        $isUserExist = "select * from blockUsers where user_name = '".$request['username']."' and block_count >= 5";
        $query      = $dbConnection->query($isUserExist);
        $data       = $query->fetchAll();
        
        return $data;
    } catch (Exception $ex) {

    }
}

function getDatabaseConnection()
{
    try {
        $dbFileName = STORAGE_DIR . DS . 'logUsers.sqlite';

        if (!file_exists($dbFileName)) {
            file_put_contents($dbFileName, '', LOCK_EX);
            chmod($dbFileName, 0777);
        }

        if (!extension_loaded('pdo_sqlite')) {
            throw new Exception('PDO SQLite extension not installed!');
        }
        
        $factory = new ConnectionFactory();
        return $factory->make(array(
                    'driver' => 'sqlite',
                    'database' => STORAGE_DIR . DS . 'logUsers.sqlite',
        ));
    } catch (Exception $ex) {
        
    }
}

function createTableIfNotExists() 
{
    try {
       $dbConnection = getDatabaseConnection();
        $sql = 'CREATE TABLE IF NOT EXISTS `blockUsers` (
                `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
                `user_name` INTEGER NOT NULL,
                `ip_address` INTEGER NOT NULL,
                `block_count` INTEGER NOT NULL,
                `created_at` DATETIME NOT NULL,
                `unblocked_at` DATETIME NOT NULL
                )';

        $dbConnection->query($sql);
        
        return true;
    } catch (Exception $ex) {
        return false;
    }
}
