<?php

$library_dir = dirname(__DIR__) . DIRECTORY_SEPARATOR . 'library';
require_once $library_dir . DIRECTORY_SEPARATOR . 'bootstrap.php';

Bootstrap::initialize('admin');

use Admin\Library\Auth;
use Application\Config;
use Application\Helper\Provider;
use Application\Helper\Security;
use Application\Request;
use Application\Session;
use Lazer\Classes\Database;
use Database\Connectors\ConnectionFactory;
use Admin\Controller\UrlPermissionController;
use Application\Http;

$postAPIData = Request::form()->all();
//print_r($postAPIData);die('aa');
$syncdata = new DataSync();
$res = $syncdata->init($postAPIData);

class DataSync
{

    private $payload = array();
    private $actionInformation = array();
    protected $response, $currentUserEmail, $currentUserIP, $currentUserName;

    public function init($payload)
    {
        $this->updatePayload($payload['payload']);
        $this->currentUserEmail = $payload['currentUserEmail'];
        $this->currentUserIP = $payload['currentUserIP'];
        $this->currentUserName = $payload['currentUserName'];



        unset($payload['currentUserEmail'], $payload['currentUserIP']);

        $this->payload = $payload;
        if (!$this->checkCredentials())
        {
            $this->showResponse(false, 'Too few arguments');
        }

        if (!$this->authenticate() && empty($payload['is_cloud']))
        {
            $this->showResponse(false, 'Unauthorized Access');
        }

        if(!empty($this->payload['callback'])){
            if(method_exists($this, $this->payload['callback'])){
                if($this->{$this->payload['callback']}()){
                   return $this->showResponse(true, 'Successfully Updated');
                }
            }
            return $this->showResponse(false, 'Something went wrong to save the data.');
        }

        if(!empty($this->payload['payload']['getStorage']))
        {
            $storageData = array();
            if(!empty($this->payload['payload']['storageSection']))
            {
                $value = $this->payload['payload']['storageSection'];
                $storageConfig = file_get_contents('../storage/admin/'.$value.'.data.json');
                $storageData[$value] = $storageConfig;
            }
            else{
                $allowedDataArray = array('settings', 'crms','campaigns', 'configurations', 
                'pixels', 'extensions', 'extensionsConfig', 'cms', 'affiliates', 
                'autoresponders', 'coupons', 'rotators', 'routings', 'splittest', 'crons');
               
                foreach ($allowedDataArray as $key => $value) {
                   $storageConfig = file_get_contents('../storage/admin/'.$value.'.data.json');
                   $storageConfig = json_decode($storageConfig, true);
                   $storageData[$value] = $storageConfig;
                }
            }
            echo json_encode($storageData);die;
        }

        if(!empty($this->payload['payload']['bulkSave']))
        {
            $bulkSaveResponse = $this->saveAndBackup($this->payload['payload']['storageSection'], $this->payload['payload']['storageSectionData']);
            echo json_encode(array('success' => true));
            die;
        }

        $this->setFormParams();
        $this->createLog();
        $this->saveData();
    }

    public function removeParams(...$arrays)
    {
        foreach ($arrays as $value)
        {
            Request::form()->remove($value);
        }
    }

    public function snakecaseToCamelcase($snakecase, $class = false)
    {
        $parts = explode('-', $snakecase);
        $camelcase = implode('', array_map('ucfirst', $parts));
        if ($class)
        {
            return $camelcase;
        }
        return lcfirst($camelcase);
    }

    public function versionControl($table_name, $id)
    {
        if(!is_numeric($id))
        {
            $this->backupExtensionData('extensionsConfig', $id);
            return;
        }
        $database_file_path = LAZER_DATA_PATH . DS . $table_name . '.data.json';
        $backup_file_path = LAZER_BACKUP_PATH . DS . $table_name . '-' . date('YmdHis') . '.data.json';
        if (!file_exists($database_file_path))
        {
            return;
        }
        $prev_content = md5(json_encode(
                        Database::table($table_name)
                                ->where('id', '=', $id)->findAll()->asArray()[0]
        ));
        $curr_content = md5(json_encode(Request::form()->all()));
        if ($curr_content == $prev_content)
        {
            return;
        }

        if (!file_exists(LAZER_BACKUP_PATH))
        {
            mkdir(LAZER_BACKUP_PATH);
        }
        /* delete of backup file number is 10 or more */
        $bkFiles = array();
        foreach (glob(LAZER_BACKUP_PATH . DS . $table_name . '-*.data.json') as $file)
        {
            $time = str_replace($table_name . '-', "", basename($file, ".data.json"));
            $bkFiles[$time] = basename($file);
        }
        krsort($bkFiles, SORT_NUMERIC);
        if (sizeof($bkFiles) == 10)
        {
            $i = 0;
            foreach ($bkFiles as $key => $value)
            {
                $i = $i + 1;
                if ($i == 10)
                {
                    unlink(LAZER_BACKUP_PATH . DS . $value);
                }
            }
        }
        /**/
        if (is_writable(LAZER_BACKUP_PATH))
        {
            if (!copy($database_file_path, $backup_file_path))
            {
                echo "Backup error!";
                exit(1);
            }
        }
    }

    public function backupExtensionData($table_name, $id)
    {
        $database_file_path = LAZER_DATA_PATH . DS . $table_name . '.data.json';
        $backup_file_path   = LAZER_BACKUP_PATH . DS . $table_name . '-' . date('YmdHis') . '.data.json';
        if (!file_exists($database_file_path)) {
            return;
        }
        
        $oldBackupFiles = array_reverse(glob(LAZER_BACKUP_PATH . DS . $table_name . '-*.data.json'));
        $oldData = json_decode(file_get_contents($oldBackupFiles[0]), true);
        if(!empty($oldData))
        {
            foreach ($oldData as $key => $value)
            {
                if((string)$id == (string)$key)
                {
                    $prev_content = $value;
                    break;
                }
            }
        }
        $curr_content = Config::extensionsConfig($id);

        $postData = Request::form()->all();
        
        if(isset($postData['extension_slug']))
        {
            return;
        }
        $prev_content = md5(json_encode($prev_content));
        $curr_content = md5(json_encode($curr_content));
        if($curr_content == $prev_content)
        {
            return;
        }

        if (!file_exists(LAZER_BACKUP_PATH)) {
            mkdir(LAZER_BACKUP_PATH);
        }
        /* delete of backup file number is 10 or more */
        $bkFiles = array();
        foreach (glob(LAZER_BACKUP_PATH . DS . $table_name . '-*.data.json') as $file) {
            $time           = str_replace($table_name . '-', "", basename($file, ".data.json"));
            $bkFiles[$time] = basename($file);
        }
        krsort($bkFiles, SORT_NUMERIC);
        if (sizeof($bkFiles) == 10) {
            $i = 0;
            foreach ($bkFiles as $key => $value) {
                $i = $i + 1;
                if ($i == 10) {
                    unlink(LAZER_BACKUP_PATH . DS . $value);
                }
            }
        }
        /**/
        if (is_writable(LAZER_BACKUP_PATH)) {
            if (!copy($database_file_path, $backup_file_path)) {
                echo "Backup error!";
                exit(1);
            }
        }
    }

    public function getDatabaseConnection()
    {
        $factory = new ConnectionFactory();
        return $factory->make(array(
                    'driver' => 'sqlite',
                    'database' => STORAGE_DIR . DS . 'adminlogs.sqlite',
        ));
    }

    public function setFormParams()
    {

        
        if (!empty($this->payload['payload']) && is_array($this->payload['payload']))
        {

            if($this->payload['payload'][0])
            {

                $line = $this->payload['payload'];
                $product_array = array(array(

                    'product_id' => (int)$line[7], 'product_price' => (int)$line[8],
    
                    'retail_price' => (int)$line[9], 'product_quantity' => (int)$line[10], 'rebill_product_price' => (int)$line[11],
    
                    'product_schedule' => '', 'product_schedule_quantity' => '', 'product_key' => '', 'nmi_plan_id' => '', 'product_name' => $line[12]
    
                ));

                Request::form()->set('product_array', json_encode(($product_array)));
                Request::form()->set('campaign_label', $line[1]);
                Request::form()->set('billing_type', $line[0]);
                Request::form()->set('crm_id', $line[2]);
                Request::form()->set('campaign_id', $line[3]);
                Request::form()->set('shipping_id', $line[4]);
                Request::form()->set('shipping_price', $line[5]);
                Request::form()->set('campaign_type', $line[6]);
                Request::form()->set('offer_id', $line[13]);
                Request::form()->set('billing_model_id', $line[14]);




            }
            else{
                foreach ($this->payload['payload'] as $key => $value)
                {
                    Request::form()->set($key, $value);
                }
            }

    
        }

        $this->removeParams('payload', 'apiToken', 'apiSecret', 'module', 'slug', 'args');
    }

    public function checkCredentials()
    {
        if (
                empty($this->payload['apiToken'])
        )
        {
            return false;
        }
        return true;
    }

    public function authenticate()
    {
        $domain = Provider::removeSubDomain(trim(Request::getHttpHost(), '/'));
        $licenseKey = $this->payload['apiToken'];
        $isValidLisence = Security::isValidLicenseKey($domain, $licenseKey);
        if (empty($isValidLisence))
        {
            return false;
        }
        return true;
    }

    public function showResponse($status, $message)
    {
        echo json_encode(array(
            'success' => $status,
            'message' => $message
        ));
        exit(0);
    }

    public function createLog()
    {
        if ($this->payload['module'] === 'Resource')
        {
            return;
        }
        $this->actionInformation['table_name'] = $this->payload['module'];
        $this->actionInformation['method_name'] = $this->payload['slug'];
        $this->actionInformation['class_name'] = sprintf('Admin\Controller\%sController', ucfirst($this->payload['module']));
        $this->actionInformation['arguments'] = !empty($this->payload['args']) ? $this->payload['args'] : [];

        if($this->payload['module'] == 'adminlogs'){
            $this->actionInformation['class_name'] = sprintf('Admin\Controller\%sController', 'AdminLogs');
        }

        if($this->payload['module'] == 'splittest'){
            $this->actionInformation['class_name'] = sprintf('Admin\Controller\%sController', 'SplitTest');
        }

        if($this->payload['module'] == 'systemlogs'){
            $this->actionInformation['class_name'] = sprintf('Admin\Controller\%sController', 'SystemLogs');
        }

        $logMethods = array('edit', 'add', 'delete', 'addManageData', 'editManageData', 'downloadExtension', 'uninstallExtension');
        
        if (in_array($this->actionInformation['method_name'], $logMethods))
        {
            $this->versionControl(
                    $this->actionInformation['table_name'], $this->actionInformation['arguments'][0]
            );
        }

        $extensions = Config::extensions();
        $result['extensionadminLoggerActive'] = false;
        foreach ($extensions as $extension)
        {
            if ($extension['extension_slug'] !== 'AdminLogs')
            {
                continue;
            }
            if ($extension['active'] === true)
            {
                $result['extensionadminLoggerActive'] = true;
            }
            break;
        }

        if (in_array($this->actionInformation['method_name'], $logMethods) && $result['extensionadminLoggerActive'])
        {
            try
            {
                $connection = $this->getDatabaseConnection();
                $method_name = $this->actionInformation['method_name'];
                $logs = $this->actionInformation['table_name'];
                if (preg_match("/edit/", $this->actionInformation['method_name']))
                {
                    $method_name = 'edit';
                }

                if ($method_name != 'add')
                {
                    $logs = $this->actionInformation['table_name'] . ' (Edited - ' . @$this->actionInformation['arguments'][0] . ')';
                }

                $connection->table('adminlogs')->insert(array(
                    'event' => $method_name,
                    'logs' => $logs,
                    'email' => !empty($this->currentUserEmail) ? $this->currentUserEmail : $this->currentUserName,
                    'ipAddress' => !empty($this->currentUserIP) ? $this->currentUserIP : '',
                    'created_on' => date('Y-m-d H:i:s', time()),
                ));
            }
            catch (Exception $ex)
            {
                
            }
        }
    }

    public function saveData()
    {
        if ($this->payload['module'] === 'Resource')
        {
            $this->getResources();
        }
        else
        {
            $formPostData = Request::form()->all();
            if(strtolower($this->actionInformation['class_name']) == 'thirdpartyextensions')
            {
               $this->actionInformation['class_name'] = 'ThirdPartyExtensions';
            }
            if(
                strtolower($this->actionInformation['table_name']) == 'extensions' && 
                in_array($this->actionInformation['method_name'], array('addManageData', 'editManageData'))
            )
            {
               Request::form()->remove('currentUserEmail');
               Request::form()->remove('currentUserIP');
               Request::form()->remove('checkURL');
            }

            header("Content-Type: application/json");
            echo json_encode(call_user_func_array(
                            array(
                                new $this->actionInformation['class_name']($this->actionInformation['arguments']),
                                $this->actionInformation['method_name'],
                            ), $this->actionInformation['arguments']
            ));
        }
    }

    public function updatePayload(&$payload)
    {
        if (!empty($payload))
        {
            foreach ($payload as $k => &$value)
            {
                if(!is_array($value))
                {
                    if ($value == "true")
                    {
                        $value = true;
                    }
                    elseif ($value == "false")
                    {
                        $value = false;
                    }
                }
                else{
                    $this->updatePayload($value);
                }
                
            }
        }
    }

    public function getResources()
    {
        if (!empty($this->payload['payload']) && !is_array($this->payload['payload']))
        {

            if (in_array($this->payload['payload'], array(
                        'fs',
                        'configurations',
                        'cron_path',
                        'googlepagespeedapiexist',
                        'googlepagespeed'
                    )))
            {
                $this->{$this->payload['payload']}();
            }
            else
            {
                $_GET['method'] = $this->payload['payload'];
                require_once sprintf(
                                '%s%sadmin%sbackend%sresource.php', dirname(__DIR__),
                                DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR
                );
            }
        }
    }

    protected function fs()
    {
        $fileSystem = array(
            array(
                '.htaccess',
                '../.htaccess',
                'Rewrite rules, caching, server enhancements'
            ),
            array(
                'storage',
                '../storage/',
                'storage'
            ),
            array(
                'extensions',
                '../extensions/',
                'extensions'
            ),
            array(
                'assets/dist',
                '../assets/dist/',
                'storage'
            ),
            array(
                'tmp',
                '../tmp/',
                'tmp'
            )
        );
        $this->response = array();
        foreach ($fileSystem as $file)
        {
            $this->response[] = array(
                'status' => is_writable($file[1]) ? true : false,
                'file' => $file[0],
                'details' => Request::getOfferPath() . $file[0],
                    // 'details' => $file[2]
            );
        }
        $this->respond();
    }

    protected function cron_path()
    {
        $this->response[] = realpath('../process-delayed-upsell.php');
        $this->respond();
    }

    protected function configurations()
    {
        $config = array(
        'license_key' => array(1 , 'Add API Key', 'license_key', 'license_key'),
        'domain' => array(1, 'Add Domain', 'general_settings', 'domain'),
        'offer_path' => array(1, 'Add offer path.', 'general_settings', 'offer_path'),
        'customer_support_email' => array(1, 'Add Customer Support Email', 'support_settings', 'support_email'),
        'customer_service_number' => array(1 , 'Add Customer Service Number', 'support_settings', 'support_number'),
        'app_prefix' => array(1 , 'Add app prefix')
    );
        
        $file = file_get_contents('../storage/admin/settings.data.json');
        $settingsconfig = json_decode($file, true);
        $this->response = array();

        foreach ($settingsconfig[0] as $key => $value)
        {
            if (!is_bool($value) && !is_null($value) && isset($config[$key]) && empty($value))
            {
                $this->response[] = array(
                    'key' => $config[$key][1],
                    'importance' => $config[$key][0],
                    'section' => $config[$key][2],
                    'highlight' => $config[$key][3]
                );
            }
        }
        $this->respond();
    }

    protected function googlepagespeedapiexist()
    {
        $file = file_get_contents('../storage/admin/settings.data.json');
        $config = json_decode($file, true);

        $this->response = array();

        if (empty($config[0]['domain']))
        {

            $this->response = array(
                'status' => false,
                'message' => 'Domain not found.'
            );
        }
        else
        {
            $this->response = array(
                'status' => true,
                'message' => 'Google PageSpeed is ready to perform.'
            );
        }
        $this->respond();
    }

    protected function googlepagespeed()
    {
        $loadFromCache = Request::get('loadChache');
        set_time_limit(0);

        $file = file_get_contents('../storage/admin/settings.data.json');
        $config = json_decode($file, true);

        $mobileVersionExist = $config[0]['enable_mobile_version'];
        $mobileVersionOnly = $config[0]['mobile_version_only'];

        $this->response = array();

        if (empty($config[0]['domain']))
        {

            $this->response = array(
                'status' => false,
                'message' => 'Domain not found.'
            );
            $this->respond();
            return;
        }


        $offerUrl = sprintf('%s/', rtrim(Request::getOfferUrl(), '/'));

        if (strpos($offerUrl, 'localhost'))
        {
            $this->response = array(
                'status' => true,
                'speed' => array(
                    array(
                        "key" => "desktop",
                        "value" => '90'
                    ),
                    array(
                        "key" => "mobile",
                        "value" => '90'
                    )
                ),
                'message' => 'PageSpeed Found'
            );
            $this->respond();
            return;
        }

        $speedData = array();

        $table = array(
            'name' => 'pagespeed',
            'attr' => array(
                'id' => 'integer',
                "desktop" => "string",
                "mobile" => "string",
                "last_modified" => "string"
            ),
        );

        //Check if data is already stored recently or not
        $cachePageSpeedfile = file_get_contents('../storage/admin/pagespeed.json');
        $storedPageSpeedData = json_decode($cachePageSpeedfile, true);

        // Check the time diff is more than 2 minutes or not
        $timeDiff = 2; // In Minutes
        if ($loadFromCache && !empty($storedPageSpeedData[0]) && ((time() - strtotime($storedPageSpeedData['last_modified'])) / 60) < $timeDiff)
        {

            unset($storedPageSpeedData['last_modified']);
            $speedData = $storedPageSpeedData;
        }
        else
        {

            foreach (["desktop", "mobile"] as $strategy)
            {

                if (!$mobileVersionExist && !strncmp($strategy, 'mobile', strlen('mobile')))
                    continue;

                if ($mobileVersionOnly && !strncmp($strategy, 'desktop', strlen('desktop')))
                    continue;

                $queryParams = array(
                    'url' => $offerUrl,
                    'strategy' => $strategy,
                );

                $queryString = http_build_query($queryParams);
                $apiEndpoint = 'https://www.googleapis.com/pagespeedonline/v5/runPagespeed';
                $url = sprintf(
                        '%s/?%s', $apiEndpoint, $queryString
                );
                $response = Http::get($url);
                $response = json_decode($response, true);

                array_push($speedData, array(
                    'key' => $strategy,
                    'value' => (float) $response['lighthouseResult']['categories']['performance']['score'] * 100
                ));
            }

            $storeIncache = $speedData;
            $storeIncache['last_modified'] = date("Y-m-d H:i:s");
            file_put_contents('../storage/admin/pagespeed.json', json_encode($storeIncache));
        }

        foreach ($speedData as $metrics)
        {

            if (!strlen($metrics['value']))
            {
                $speedData[$metrics]['value'] = "0";
            }
        }

        if (!empty($speedData))
        {
            $this->response = array(
                'status' => true,
                'speed' => $speedData,
                'message' => 'PageSpeed Found'
            );
        }
        else
        {
            $this->response = array(
                'status' => false,
                'message' => 'PageSpeed not found.'
            );
        }
        $this->respond();
    }

    public function respond()
    {
        if (!empty($this->response))
        {
            echo json_encode($this->response);
        }
    }

    private function saveCustomVariables(){
        try{
           
            if($this->payload && $this->payload['payload'] && !empty($this->payload['payload']['custom_fields'])){
                 file_put_contents(STORAGE_DIR . DS . 'customfields.data.json', 
                json_encode($this->payload['payload']['custom_fields']), LOCK_EX);
                return true;
            }
           return false;
           
        } catch (Exception $ex){
            return false;
        }
    }


    private function saveAndBackup($table_name, $data)
    {
        $database_file_path = LAZER_DATA_PATH . DS . $table_name . '.data.json';
        $oldBackupFiles = array_reverse(glob(LAZER_BACKUP_PATH . DS . $table_name . '-*.data.json'));
        $prev_content = file_get_contents($oldBackupFiles[0]);
        $curr_content = $data;
       
        $backup_file_path   = LAZER_BACKUP_PATH . DS . $table_name . '-' . date('YmdHis') . '.data.json';
        if (!file_exists($database_file_path)) {
            return;
        }
        
        
        $prev_content = md5(json_encode($prev_content));
        $curr_content = md5(json_encode($curr_content));
        if ($curr_content == $prev_content) {

            return;
        }

        if (!file_exists(LAZER_BACKUP_PATH)) {
            mkdir(LAZER_BACKUP_PATH);
        }
        /* delete of backup file number is 10 or more */
        $bkFiles = array();
        foreach (glob(LAZER_BACKUP_PATH . DS . $table_name . '-*.data.json') as $file) {
            $time           = str_replace($table_name . '-', "", basename($file, ".data.json"));
            $bkFiles[$time] = basename($file);
        }
        krsort($bkFiles, SORT_NUMERIC);
        if (sizeof($bkFiles) == 10) {
            $i = 0;
            foreach ($bkFiles as $key => $value) {
                $i = $i + 1;
                if ($i == 10) {
                    unlink(LAZER_BACKUP_PATH . DS . $value);
                }
            }
        }
        /**/
        if (is_writable(LAZER_BACKUP_PATH)) {
            if (!copy($database_file_path, $backup_file_path)) {
                echo "Backup error!";
                exit(1);
            }

            file_put_contents($database_file_path, $data);

            // last id update for storage migrator

            $database_config_file_path = LAZER_DATA_PATH . $table_name . '.config.json';
            $cData = file_get_contents($database_config_file_path);
            $cData = json_decode($cData, true);

            if(!empty($cData)){
                $maxValKey = array();
                $data = json_decode($data, true);
                foreach ($data as $value) {
                   array_push($maxValKey, $value['id']);
                }
                $max = max($maxValKey);
                $cData['last_id'] = (int) $max;
                $newCData = json_encode($cData);
                file_put_contents($database_config_file_path, $newCData);
            }
        }
    }

    private function saveFieldsData(){
        try{
         
           if($this->payload && $this->payload['payload'] && !empty($this->payload['payload']['fields_data'])){
                file_put_contents(STORAGE_DIR . DS . 'fields.data.json', 
               json_encode($this->payload['payload']['fields_data']), LOCK_EX);
               return true;
           }
          return false;
          
       } catch (\Exception $ex){
           return false;
       }
   }

   private function saveShieldsData(){
        try{
        
            if($this->payload && $this->payload['payload'] && isset($this->payload['payload']['shields_data'])){
                file_put_contents(STORAGE_DIR . DS . 'shields.data.json', 
                json_encode($this->payload['payload']['shields_data']), LOCK_EX);
                return true;
            }
        return false;
        
        } catch (Exception $ex){
            return false;
        }
    }

}
