<?php

namespace Application\Model;

use Application\CrmPayload;
use Application\CrmResponse;
use Application\Http;

class Layer2 extends BaseCrm
{

    private $methodSlugMapping = array(
        'prospect'             => 'v1/leads/import/',
        'newOrderWithProspect' => 'v1/order/import/',
        'newOrder'             => 'v1/order/import/',
        'newOrderCardOnFile'   => 'v1/upsell_flow/',
        'initSession'          => 'v1/init/',
        'importClick'          => 'v1/clicks/import/',
        'importUpsell'         => 'v1/upsell_flow/'
    );

    const LAYER2_ERROR = 'Please try again later';

    public function __construct($crmId)
    {
        parent::__construct($crmId);
    }

    protected function beforeAnyCrmClassMethodCall()
    {
        $this->params = $this->response = array();
    }

    protected function prospect()
    {
        $this->params = array_replace($this->params, CrmPayload::all());

        $this->prepareShippingDetails();
        $this->prepareAffiliates();

        $this->params['slug'] = $this->methodSlugMapping['prospect'];

        if ($this->makeHttpRequest() === false) {
            return false;
        }

        CrmResponse::replace(array(
            'success'    => true,
            'prospectId' => $this->response['customerId'],
        ));
    }

    protected function newOrderWithProspect()
    { 

        $this->params = array_replace($this->params, CrmPayload::all());

        $this->prepareShippingDetails();
        $this->prepareBillingDetails();
        $this->prepareCardDetails();
        $this->prepareAffiliates();

        $this->params['customerID']   = $this->params['prospectId'];        
        $this->params['slug'] = $this->methodSlugMapping['newOrderWithProspect'];
    
        $this->unsetExtraParams();

        if ($this->makeHttpRequest() === false) {
            return false;
        }

        if($this->response['firePixel'] == '0') {
            $skipFire = true; 
        }

        CrmResponse::replace(array(
            'success'        => true,
            'orderId'        => $this->response['transactionId'],
            'customerId'     => $this->response['customerId'],
            'subscriptionId' => $this->response['subscriptionId'],
            'crmOrderId'     => !empty($this->response['crmOrderId']) ? $this->response['crmOrderId'] : '',
            'crmTransactionId' => !empty($this->response['crmTransactionId']) ? $this->response['crmTransactionId'] : '',
            'skipFire' => !($skipFire) ? false : true
        ));
    }

    protected function newOrder()
    {
        
        $this->params = array_replace($this->params, CrmPayload::all());

        $this->prepareShippingDetails();
        $this->prepareBillingDetails();
        $this->prepareCardDetails();
        $this->prepareAffiliates();

        if(!empty($this->params['partialOrderID']))
        {
            $this->params['orderId'] = $this->params['partialOrderID'];
            unset($this->params['partialOrderID']);
        }
        else{
            unset($this->params['orderId']);
        }

        unset($this->params['parentOrderId']);
        $this->params['slug']      = $this->methodSlugMapping['newOrder'];

        $this->unsetExtraParams();

        if ($this->makeHttpRequest() === false) {
            return false;
        }

        if($this->response['firePixel'] == '0') {
            $skipFire = true; 
        }

        CrmResponse::replace(array(
            'success'        => true,
            'orderId'        => $this->response['transactionId'],
            'customerId'     => $this->response['customerId'],
            'subscriptionId' => $this->response['subscriptionId'],
            'crmOrderId'     => !empty($this->response['crmOrderId']) ? $this->response['crmOrderId'] : '',
            'crmTransactionId' => !empty($this->response['crmTransactionId']) ? $this->response['crmTransactionId'] : '',
            'skipFire' => !($skipFire) ? false : true,
        ));
    }

    protected function newOrderCardOnFile()
    {
        return $this->importUpsell();
    }

    protected function importUpsell()
    {
        $this->params = array_replace($this->params, CrmPayload::all());

        $this->prepareAffiliates();
        $this->unsetExtraParams();
        
        $this->params['slug'] = $this->methodSlugMapping['importUpsell'];   
    
        if ($this->makeHttpRequest() === false) {
            return false;
        }

        if($this->response['firePixel'] == '0') {
            $skipFire = true; 
        }

        CrmResponse::replace(array(
            'success'    => true,
            'subscriptionId' => $this->response['subscriptionId'],
            'crmOrderId'     => !empty($this->response['crmOrderId']) ? $this->response['crmOrderId'] : '',
            'crmTransactionId' => !empty($this->response['crmTransactionId']) ? $this->response['crmTransactionId'] : '',
            'skipFire' => !($skipFire) ? false : true,
        ));
    }

    public function initSession() 
    {
        $this->beforeAnyCrmClassMethodCall();
        $this->params = array_replace($this->params, CrmPayload::all());

        $this->prepareAffiliates();

        $this->params['slug'] = $this->methodSlugMapping['initSession'];

        if ($this->makeHttpRequest() === false) {
            return;
        }
        
    }

    public function importClick()
    {
        $this->beforeAnyCrmClassMethodCall();
        $this->params = array_replace($this->params, CrmPayload::all());

        $this->params['slug'] = $this->methodSlugMapping['importClick'];

        if ($this->makeHttpRequest() === false) {
            return;
        }
    }

    private function beforeHttpRequest()
    {

        $this->params['apiKey'] = $this->username;
        if (!empty($this->params['forceGatewayId'])) {
            $this->params['forceMerchantId'] = trim($this->params['forceGatewayId']);
        }

        unset(
            $this->params['forceGatewayId'], $this->params['preserveGateway'],
            $this->params['parentOrderId'], $this->params['userIsAt'], $this->params['billingSameAsShipping']
        );

        $metaKeys = preg_grep('/^meta\..+$/', array_keys($this->params));
        foreach ($metaKeys as $metaKey) {
            unset($this->params[$metaKey]);
        }

        $this->response = array();
    }

    private function makeHttpRequest()
    {

        $this->beforeHttpRequest();

        $url = $this->getUrl($this->params['slug']);
        unset($this->params['slug']);

        $params = $this->params;

        $response = Http::post($url, http_build_query($params));

        if (!empty($response['curlError'])) {
            CrmResponse::replace(array(
                'success' => false,
                'errors'  => array(
                    'curlError' => $response['errorMessage'],
                ),
            ));
            return false;
        }

        return $this->prepareResponse($response);
    }

    private function prepareResponse($response)
    {
        $this->response = json_decode($response, true);
        
        if (!empty($this->response['statusCode']) && $this->response['statusCode'] == 200) {
            CrmResponse::update(array(
                'success'        => true,
                'response'       => $this->response,
            ));
            return true;
        }

        if (empty($this->response['result']) || $this->response['result'] !== 'SUCCESS') {
            if (is_array($this->response['statusMessage'])) {
                $this->response['statusMessage'] = sprintf(
                    '%s: %s', key($this->response['statusMessage']),
                    $this->response['statusMessage'][key($this->response['statusMessage'])]
                );
            }

            if ($this->response['statusCode'] == 400 && empty($this->response['statusMessage'])) {
                $this->response['statusMessage'] = self::LAYER2_ERROR;
            }
            
            CrmResponse::replace(array(
                'success'          => false,
                'errors'           => array(
                    'crmError' => $this->response['statusMessage'],
                ),
            ));

            return false;
        }

        return true;
    }

    private function getUrl($slug)
    {
        if ($slug == 'v1/init/') {
            $slug = $slug . '?id=' . CrmPayload::get('campaignId');
        }
        elseif ($slug == 'v1/upsell_flow/') {
            $rsid = CrmPayload::get('meta.mainStepData');
            $slug = $slug . '?id=' . CrmPayload::get('campaignId').'&rsid='.$rsid['subscriptionId'];
            unset($this->params['gatewayId']);
        }
        elseif ($slug == 'v1/clicks/import/') {
            $slug = $slug;
            unset($this->params['gatewayId']);
        }
        else{
            $slug = $slug . '?id=' . CrmPayload::get('gatewayId');
            unset($this->params['gatewayId']);
            if(!empty($this->params['enableProspect'])){
                $slug = $slug . CrmPayload::get('campaignId');
                unset($this->params['enableProspect']);
            } 
        }

        return sprintf('%s/%s', $this->endpoint, $slug);
    }

    private function unsetExtraParams()
    {
        if($this->params['meta.crmMethod'] == 'importUpsell'){
            unset($this->params['sessionId'], $this->params['customerId'], $this->params['previousOrderId'], $this->params['previousOrderId'], $this->params['newSubscription'], $this->params['ipAddress'], $this->params['cvv'], $this->params['cardExpiryYear'], $this->params['cardExpiryMonth'], $this->params['cardNumber'], $this->params['billingCountry'], $this->params['billingState'], $this->params['billingCity'], $this->params['billingZip'], $this->params['billingAddress2'], $this->params['billingAddress1'], $this->params['billingLastName'], $this->params['billingFirstName'], $this->params['billingSameAsShipping'], $this->params['userIsAt'], $this->params['userAgent'], $this->params['shippingCountry'], $this->params['shippingState'], $this->params['shippingCity'], $this->params['shippingZip'], $this->params['shippingAddress2'], $this->params['shippingAddress1'], $this->params['phone'], $this->params['email'], $this->params['lastName'], $this->params['firstName'], $this->params['cardType'], $this->params['paySource'], $this->params['campaignId'], $this->params['prospectId'], $this->params['products']);

        }
        unset($this->params['cardType'], $this->params['paySource'], $this->params['campaignId'], $this->params['prospectId'], $this->params['products']);

    }

    private function prepareShippingDetails()
    {
        $shippingDetails = array(
            'emailAddress' => $this->accessor->getValue($this->params, '[email]'),
            'phoneNumber'  => $this->accessor->getValue($this->params, '[phone]'),
            'address1'     => $this->accessor->getValue($this->params, '[shippingAddress1]'),
            'address2'     => $this->accessor->getValue($this->params, '[shippingAddress2]'),
            'postalCode'   => $this->accessor->getValue($this->params, '[shippingZip]'),
            'city'         => $this->accessor->getValue($this->params, '[shippingCity]'),
            'state'        => $this->accessor->getValue($this->params, '[shippingState]'),
            'country'      => $this->accessor->getValue($this->params, '[shippingCountry]'),
        );

        unset(
            $this->params['email'], $this->params['phone'],
            $this->params['shippingAddress1'], $this->params['shippingAddress2'],
            $this->params['shippingZip'], $this->params['shippingCity'],
            $this->params['shippingState'], $this->params['shippingCountry']
        );

        $this->params = array_replace($this->params, array_filter($shippingDetails));
    }

    private function prepareBillingDetails()
    {
        $billingDetails = array('billShipSame' => '1');

        if (strtolower($this->params['billingSameAsShipping']) !== 'yes') {
            $billingDetails = array(
                'billShipSame'       => '0',
                'billingAddress1'    => $this->accessor->getValue($this->params, '[billingAddress1]'),
                'billingAddress2'    => $this->accessor->getValue($this->params, '[billingAddress2]'),
                'billingPostalCode'  => $this->accessor->getValue($this->params, '[billingZip]'),
                'billingCity'        => $this->accessor->getValue($this->params, '[billingCity]'),
                'billingState'       => $this->accessor->getValue($this->params, '[billingState]'),
                'billingCountry'     => $this->accessor->getValue($this->params, '[billingCountry]')
            );
        }

        unset(
            $this->params['billingFirstName'], $this->params['billingLastName'],
            $this->params['billingAddress1'], $this->params['billingAddress2'],
            $this->params['billingCity'], $this->params['billingState'],
            $this->params['billingZip'], $this->params['billingCountry']
        );

        $this->params = array_replace($this->params, $billingDetails);
    }

    private function prepareCardDetails()
    {
        $cardDetails = array(
            'cardType'         => $this->accessor->getValue($this->params, '[cardType]'),
            'cardMonth'        => $this->accessor->getValue($this->params, '[cardExpiryMonth]'),
            'cardYear'         => !empty($this->params['cardExpiryYear']) ? '20'.$this->accessor->getValue($this->params, '[cardExpiryYear]') : '',
            'cardSecurityCode' => $this->accessor->getValue($this->params, '[cvv]'),
            'paySource'        => $this->accessor->getValue($this->params, '[paySource]'),
        );
        
        if(empty($cardDetails['paySource']))
        {
            $cardDetails['paySource'] = 'CREDITCARD';
        }

        $cardDetails['cardType'] = strtoupper($cardDetails['cardType']);
        if ($cardDetails['cardType'] === 'MASTER') {
            $cardDetails['cardType'] .= 'CARD';
        }
        
        if(!empty($this->params['cardType']) && $this->params['cardType'] == 'COD')
        {
            $cardDetails['paySource'] = 'COD';
            unset($cardDetails['cardType']);
            unset($this->params['cardType']);
        }
        
        if(!empty($this->params['cardType']) && $this->params['cardType'] == 'DIRECTDEBIT')
        {
            unset($cardDetails['cardType']);
            unset($this->params['cardType']);
        }

        unset(
            $this->params['cardExpiryMonth'], $this->params['cardExpiryYear'], $this->params['cvv']
        );

        $this->params = array_replace($this->params, array_filter($cardDetails));
    }

    private function prepareAffiliates()
    {
        $affiliates = $this->params['affiliates'];

        $affiliateParams = array_filter(array(
            'aff'        => $this->accessor->getValue($affiliates, '[affId]'),
            'subAff'     => $this->accessor->getValue($affiliates, '[sId]'),
            'c1'         => $this->accessor->getValue($affiliates, '[c1]'),
            'c2'         => $this->accessor->getValue($affiliates, '[c2]'),
            'c3'         => $this->accessor->getValue($affiliates, '[c3]'),
            'c4'         => $this->accessor->getValue($affiliates, '[c4]'),
            'c5'         => $this->accessor->getValue($affiliates, '[c5]'),
        ));

        unset($this->params['affiliates']);

        $this->params = array_replace($this->params, $affiliateParams);
    }
    
    public static function isValidCredential($credential)
    {
        return true;
    }

}
