<?php

namespace Application\Model;

use Application\Config;
use Application\Http;
use Application\Logger;
use Application\Request;
use Application\Session;
use Detection\MobileDetect;
use Application\Helper\Provider;
use Exception;
use DateTime;
use Database\Connectors\ConnectionFactory;

class Pixel
{
    const API_ENDPOINT = 'https://www.google-analytics.com/mp/collect?measurement_id=';
    const TIK_TOK_API_ENDPOINT = 'https://business-api.tiktok.com/open_api/v1.2/pixel/track/';

    protected $gaFourPayLoad = array(
        'client_id' => 'client_id',
        'events'    => array(
            0 => array(
                'name'   => 'name',
                'params' => array(
                    'currency'       => 'currency',
                    'transaction_id' => 'transaction_id',
                    'value'          => 'value',
                    'items'          => array(                      
                    ),
                ),
            ),
        ),
    );
    protected $tiktokPayload = array(
        'pixel_code' => 'pixelID',
        'event' => 'eventName',
        'event_id' => 'event_id',
        'timestamp' => 'timestamp',
        'test_event_code' => 'test_event_code',
        'context' => array(
            'ad' => array(
                'callback' => 'ttclid',
            ),
            'page' => array(
                'url' => 'url',
                'referrer' => 'referrer',
            ),
            'user' => array(
               'external_id' => 'external_id',
                'phone_number' => 'phone_number',
                'email' => 'email',
            ),
            'user_agent' => 'user_agent',
            'ip' => 'ip',
        ),
        'properties' => array(
            'contents' => array(
            ),
            'currency' => 'currency',
            'value' => 'value',
        ),
    );

    
    private $previousStepId, $currentStepId, $previousConfigId, $currentConfigId;
    private $currentStepPixels;
    private $nonConversionFBEvents = array('PageView', 'ViewContent', 'AddToCart');
    private $nonConversionGA4Events = array('generate_lead', 'add_payment_info', 'begin_checkout', 'view_cart', 'add_to_cart', 'remove_from_cart');
    private $nonConversionTikTokEvents = array('AddPaymentInfo', 'InitiateCheckout', 'ViewContent', 'AddToCart');
    private $manualPixels = array();
    private $conversionPixelEvents = ['Conversion Pixel (HTML)','Conversion Pixel (Server to Server)','Conversion Pixel (HTML+Postback)','FB Conversion Pixel (Server)','Tik Tok Conversion Pixel (Server)'];

    public function __construct()
    {
        $this->currentStepPixels = array();
        $this->currentStepId     = (int) Session::get('steps.current.id');
        $this->currentConfigId   = (int) Session::get('steps.current.configId');
        if (Session::has('steps.previous.configId') !== true) {
            $this->previousConfigId = (int) Session::get('steps.current.configId');
        }
        else
        {
            $this->previousConfigId = (int) Session::get('steps.previous.configId');
        }
        $this->previousStepId   = (int) Session::get('steps.previous.id');
        
        $this->loadPixelsByConfigId($this->previousConfigId);

        if($this->currentStepId > 1)
        {
            $conversionPixels = $this->currentStepPixels;
            $this->loadPixelsByConfigId($this->currentConfigId);
            $currentStepGeneralPixels = array();
            foreach ($this->currentStepPixels as $key => $value) {
                if(
                    ($value['pixel_type'] == 'FB Conversion Pixel (Server)' && in_array($value['pixel_events'], $this->nonConversionFBEvents )) ||
                    ($value['pixel_type'] == 'Google Analytics 4 (GA4)' && in_array($value['pixel_events_ga4'], $this->nonConversionGA4Events )) ||
                    ($value['pixel_type'] == 'Tik Tok Conversion Pixel (Server)' && in_array($value['pixel_events_tik_tok'], $this->nonConversionTikTokEvents)) ||
                     $value['pixel_type'] == 'General'
                )
                {
                    array_push($currentStepGeneralPixels, $value);
                }
            }
            $this->currentStepPixels = array_merge($conversionPixels, $currentStepGeneralPixels);
        }
        $this->tableName = 'fbpixelfallback';
        $this->tableName1 = 'postbackpixel';
    }

    public function getHeadPixelsAsHtml()
    {
        $headPixelsHtml = '';
        foreach ($this->currentStepPixels as $pixel) {
            if ($pixel['pixel_placement'] !== 'head') {
                continue;
            }
            $headPixelsHtml .= $this->parsePixel($pixel);
        }
        return $headPixelsHtml;
    }

    public function getTopPixelsAsHtml()
    {
        $topPixelsHtml = '';
        foreach ($this->currentStepPixels as $pixel) {
            if ($pixel['pixel_placement'] !== 'top') {
                continue;
            }
            $topPixelsHtml .= $this->parsePixel($pixel);
        }
        return $topPixelsHtml;
    }

    public function getBottomPixelsAsHtml()
    {
        $bottomPixelsHtml = '';
        foreach ($this->currentStepPixels as $pixel) {
            if ($pixel['pixel_placement'] !== 'bottom') {
                continue;
            }
            $bottomPixelsHtml .= $this->parsePixel($pixel);
        }
        return $bottomPixelsHtml;
    }
    
    public function setClickID()
    {

        if (Session::get('pixels.clickIdGenerated') === true) {
            return;
        }

        $this->loadPixelsByConfigId($this->currentConfigId);

        foreach ($this->currentStepPixels as $pixel) {
            if ($this->isValidDevice($pixel) === false) {
                continue;
            }

            if ($this->isValidAffiliates($pixel) === false) {
                continue;
            }

            if (!empty($pixel['click_pixel'])) {
                $clickId = $this->getClickID($pixel['click_pixel']);
                if (!empty($clickId)) {
                    Session::set('affiliates.clickId', $clickId);
                    Session::set('queryParams.click_id', $clickId);
                    Session::set('pixels.clickIdGenerated', true);
                }
            }

        }
    }

    public function hasClickPixels()
    {
        $this->loadPixelsByConfigId($this->currentConfigId);
        foreach ($this->currentStepPixels as $pixel) {
            if (!empty($pixel['click_pixel'])) {
                return true;
            }
        }
    }

    private function loadPixelsByConfigId($configId)
    {
        $this->currentStepPixels = array();
        $pixels                  = Config::pixels();
        $isRouteBin              = session::get('extensions.binmanager.is_route_bin');

        foreach ($pixels as $pixel) {
            if(!empty($pixel['enable_bin_check']) && !empty($isRouteBin)) {
                continue;
            }
            
            if (empty($pixel['fb_manual_trigger']) && empty($pixel['ga4_manual_trigger']) && empty($pixel['tik_tok_manual_trigger']) && in_array($configId, explode(',',$pixel['configuration_id']))) {
                array_push($this->currentStepPixels, $pixel);
            }
        }
        $this->updatePixelsPriority();
    }
    
    public function updatePixelsPriority()
    {
        $this->currentStepPixels = $this->multid_sort($this->currentStepPixels, 'pixel_firing_priority');;
    }
    
    public function multid_sort($arr, $index) {
        $temparr = array();
        foreach ($arr as $key => $value) {
            $temparr[$key] = !empty($value[$index]) ? $value[$index] : 1;
        }
        array_multisort($temparr, SORT_DESC, $arr);
        usort($arr, array($this, 'sortByPriority'));
        return $arr;
    }
    
    public function sortByPriority($a, $b) {
        return $a['pixel_firing_priority'] - $b['pixel_firing_priority'];
    }
    
    
    private function parsePixel($pixel)
    {
        if ($this->pixelTriggers($pixel) === false) {
            return '';
        }    

        switch ($pixel['pixel_type']) {
            
            case 'HTML':
                return $this->parseHtmlPixel($pixel);
            
            case 'Postback':
                return $this->parsePostbackPixel($pixel);
                
            case 'Postback + 3rd Party Pixels':
                return $this->parsePostback3rdPartyPixel($pixel);

            case 'Conversion Pixel (HTML)':
                return $this->parseHtmlPixel($pixel);

            case 'Conversion Pixel (Server to Server)':
                return $this->parsePostbackPixel($pixel);

            case 'Conversion Pixel (HTML+Postback)':
                return $this->parsePostback3rdPartyPixel($pixel);

            case 'Cake Postback':
                return $this->parseCakePostbackPixel($pixel);

            case 'Cake Postback + 3rd Party Pixels':
                return $this->parseCakePostback3rdPartyPixel($pixel);

            case 'HasOffers Postback':
                return $this->parseHasOffersPostbackPixel($pixel);

            case 'HasOffers Postback + 3rd Party Pixels':
                return $this->parseHasOffersPostback3rdPartyPixel($pixel);

            case 'HitPath Postback':
                return $this->parseHitPathPostbackPixel($pixel);

            case 'HitPath Postback + 3rd Party Pixels':
                return $this->parseHasOffersPostback3rdPartyPixel($pixel);
                
            case 'General':
                $isExtensionEnable = Provider::checkExtensions('TrafficLoadBalancer');
                if(!$pixel['ignore_pixel_on_order_filter'] && $isExtensionEnable) {
                    $genPixelToFire = array();
                    if( Session::has("GeneralPixelToFire")) {
                        $genPixelToFire = Session::get("GeneralPixelToFire");
                    }

                    array_push($genPixelToFire, array(
                        'content' => $this->parseHtmlPixel($pixel),
                        'type' => $pixel['pixel_placement']
                    ));

                    Session::set("GeneralPixelToFire", $genPixelToFire);
                    return "";
                }

                return $this->parseHtmlPixel($pixel);
                
            case 'Decline':
                return $this->parseHtmlPixel($pixel);
                
            case 'Submission':
                return $this->parseHtmlPixel($pixel);    
                
            case 'On Decline':
                return $this->parseHtmlPixel($pixel);
                
            case 'On Form Submission':
                return $this->parseHtmlPixel($pixel);

            case 'FB Conversion Pixel (Server)':
                return $this->parseFBPostbackPixel($pixel);  
                
            case 'Google Analytics 4 (GA4)':
                return $this->parseGAFourPostbackPixel($pixel);

           case 'Tik Tok Conversion Pixel (Server)':
                return $this->parseTikTokPostbackPixel($pixel);
        }
    }

    protected function parseHitPathPostback3rdPartyPixels($pixel)
    {
        $url = $this->parseTokens($pixel['convert_pixel']);
        $this->doPostBack($pixel,$url);

        if (!empty($pixel['third_party_postback_url'])) {
            $url = $this->parseTokens(trim($pixel['third_party_postback_url']));
            $this->doPostBack($pixel,$url);
        }

        if (!empty($pixel['third_party_html'])) {
            return $this->parseTokens(trim($pixel['third_party_html']));
        }
    }

    protected function parseHitPathPostbackPixel($pixel)
    {
        $url = $this->parseTokens($pixel['convert_pixel']);
        $this->doPostBack($pixel,$url);
        return sprintf(
            '<!-- HitPath Postback Triggered%s%s%s/HitPath Postback Triggered -->',
            PHP_EOL, $url, PHP_EOL
        );
    }

    protected function parseHasOffersPostback3rdPartyPixel($pixel)
    {
        $url = $this->parseTokens($pixel['convert_pixel']);
        $this->doPostBack($pixel,$url);

        if (!empty($pixel['third_party_postback_url'])) {
            $url = $this->parseTokens(trim($pixel['third_party_postback_url']));
            $this->doPostBack($pixel,$url);
        }

        if (!empty($pixel['third_party_html'])) {
            return $this->parseTokens(trim($pixel['third_party_html']));
        }
    }

    protected function parseHasOffersPostbackPixel($pixel)
    {
        $url = $this->parseTokens($pixel['convert_pixel']);
        $this->doPostBack($pixel,$url);
        return sprintf(
            '<!-- HasOffers Postback Triggered%s%s%s/HasOffers Postback Triggered -->',
            PHP_EOL, $url, PHP_EOL
        );
    }

    protected function parseCakePostback3rdPartyPixel($pixel)
    {
        if (!empty($pixel['convert_pixel'])) {
            $url = $this->parseTokens(trim($pixel['convert_pixel']));
            $this->doPostBack($pixel,$url);
        }
        if (!empty($pixel['third_party_postback_url'])) {
            $url = $this->parseTokens(trim($pixel['third_party_postback_url']));
            $this->doPostBack($pixel,$url);
        }
        if (!empty($pixel['third_party_html'])) {
            return $this->parseTokens(trim($pixel['third_party_html']));
        }
    }

    protected function parseCakePostbackPixel($pixel)
    {
        $url = $this->parseTokens($pixel['convert_pixel']);
        $this->doPostBack($pixel,$url);
        return sprintf(
            '<!-- Cake Postback Triggered%s%s%s/Cake Postback Triggered -->',
            PHP_EOL, $url, PHP_EOL
        );
    }

    protected function parsePostback3rdPartyPixel($pixel)
    {
        if (!empty($pixel['postback_url'])) {
            $url = $this->parseTokens(trim($pixel['postback_url']));
            $this->doPostBack($pixel,$url);
        }

        if (!empty($pixel['third_party_postback_url'])) {
            $url = $this->parseTokens(trim($pixel['third_party_postback_url']));
            $this->doPostBack($pixel,$url);
        }

        if (!empty($pixel['third_party_html'])) {
            return $this->parseTokens(trim($pixel['third_party_html']));
        }

        return '';
    }

    protected function parsePostbackPixel($pixel)
    {
        $url = $this->parseTokens($pixel['postback_url']);
        $this->doPostBack($pixel,$url);
        return '';
    }

    protected function parseHtmlPixel($pixel)
    {
        return $this->parseTokens($pixel['html_pixel']);
    }

    protected function htmlPixel($pixelHtml)
    {
        return sprintf('<img src="%s" style="display: none">', $pixelHtml);
    }

    private function parseTokens($stringWithTokens)
    {
        return preg_replace_callback(
            "/\{([a-z0-9_]+)\}/i", function ($data) {

                if ($data[1] === 'order_id' || $data[1] === 'orderId') {
                    return Session::get(
                        sprintf('steps.%d.orderId', $this->previousStepId)
                    );
                }
                
                if ($data[1] === 'total_incl_tax') {
                    return Session::get(
                        sprintf('steps.%d.total_incl_tax', $this->previousStepId)
                    );
                }
                
                if ($data[1] === 'split_order_id' || $data[1] === 'splitOrderId') {
                    return Session::get(
                        sprintf('steps.%d.splitOrder.orderId', $this->previousStepId)
                    );
                }
                
                if ($data[1] === 'order_total' || $data[1] === 'orderTotal') {
                    $orderTotal = $this->getOrderTotal();
                    return $orderTotal;
                }
                
                if ($data[1] === 'campaign_id' || $data[1] === 'campaignId') {
                	$products = Session::get(
			                        sprintf('steps.%d.products', $this->previousStepId)
			                    );
                    return $products[0]['campaignId'];
                }

                if ($data[1] === 'card_type' || $data[1] === 'cardType') {
                	return Session::get('customer.cardType');
                }
                
                if (preg_match('/custom_pixel/i', $data[1])) {
                    return Session::get('custom_pixel_tokens.'.$data[1]);
                }

                $allowedTokens = array('order_id','order_total','AFID','AFFID','SID','AID','C1','C2','C3','C4','C5','C6','click_id','firstName','lastName','email','phone','shippingCity','shippingState','shippingCountry','campaign_id','card_type','split_order_id','custom_pixel_your_token_name','gclid','evclid', 'total_incl_tax');

                if(!in_array($data[1], $allowedTokens)){
                    return "{".$data[1]."}";
                }
                
                $formData = array(
                    'firstName',
                    'lastName',
                    'email',
                    'phone',
                    'shippingCity',
                    'shippingState',
                    'shippingCountry'
                );
                
                foreach($formData as $formTokens) {
                    if ($data[1] === $formTokens) {
                        return Session::get(
                            sprintf('customer.%s', $formTokens)
                        );
                    }
                }

                if ($data[1] === 'campaign_id' || $data[1] === 'campaignId') {
                	$products = Session::get(
			                        sprintf('steps.%d.products', $this->previousStepId)
			                    );
                    return $products[0]['campaignId'];
                }

                if ($data[1] === 'card_type' || $data[1] === 'cardType') {
                	return Session::get('customer.cardType');
                }

                $param = strtolower(str_replace('_', '', $data[1]));

                $affiliates = array_change_key_case(Session::get('affiliates'));

                foreach ($affiliates as $key => $value) {
                    if ($param === $key) {
                        return rawurlencode($value);
                    }
                }

            }, $stringWithTokens
        );
    }
    
    private function getOrderTotal()
    {
        
        if(Session::get('crmType') === 'twentyninenext')
        {
            $orderTotal = Session::get(
                sprintf('steps.%d.total_excl_tax', $this->previousStepId)
            );
            //$orderTotal = $transactions[0]['amount'];
            if(!empty($orderTotal))
            {
                return $orderTotal;
            }
        }

        $products = Session::get(
                sprintf('steps.%d.products', $this->previousStepId)
            );
        $orderTotal = 0.00;
        
        if(!empty($products)) 
        {
            foreach ($products as $value)
            {
                if(Session::get('crmType') === 'vrio') {
                    $orderTotal += ($value['offerPrice'] * $value['offerQuantity']) + $value['shippingPrice'];
            	} else {
                    $orderTotal += ($value['productPrice'] * $value['productQuantity']) + $value['shippingPrice'];
            	}
            }
        }
        
        return $orderTotal;
    }

    protected function doPostBack($pixel,$postbackUrl)
    {
        if(!empty($pixel) && $pixel['delay']==1){
            $dateTime_3p = new DateTime();
            if($pixel['data_process_type'] == 'days'){
                $dateTime_3p->modify(sprintf('+%d days', $pixel['process_days']));
            }
            if($pixel['data_process_type'] == 'hours'){
                $dateTime_3p->modify(sprintf('+%d hour', $pixel['process_days']));
            }
            if($pixel['data_process_type'] == 'minutes'){
                $dateTime_3p->modify(sprintf('+%d minute', $pixel['process_days']));
            }
            $scheduledAt_3p = $dateTime_3p->format('Y-m-d H:i:s');
            try {
                $this->createDatabase1(); 
                $connection = $this->getDatabaseConnection1();
                $connection->table($this->tableName1)->insert(array(
                    'pixel_id'      => $pixel['id'],
                    'pixel_type'      => $pixel['pixel_type'],
                    'data_process_type'  => $pixel['data_process_type'],
                    'process_days'  => $pixel['process_days'],
                    'postback_url'  => $pixel['postback_url'],
                    'http_response'  => 0,
                    'status'        => 0,            
                    'created_at'    => date('Y-m-d H:i:s', time()),
                    'updated_at'    => date('Y-m-d H:i:s', time()),
                    'scheduled_at'  => $scheduledAt_3p
                ));
            } catch (Exception $ex) {
            }
        }
        else{
            $response = Http::get(
                $postbackUrl, array(), array(CURLOPT_CONNECTTIMEOUT => 5)
            );
            if (DEV_MODE) {
                Logger::write('postBack', $response);
                Session::set('postBack_request.'.$this->previousStepId, $postbackUrl);
                Session::set('postBack_response.'.$this->previousStepId, $response);
            }
            if (!empty($response['curlError'])) {
                return '';
            }
            return $response;
        }
        
        
    }
    
    protected function getClickID($url)
    {
        $response = $this->doPostBack(array(),$this->parseTokens($url));
        if (!empty($response['curlError'])) {
            return '';
        }

        $data = explode('=', $response);
        if (!empty($data) && is_array($data) && !empty($data[1])) {
            return $data[1];
        }

        return '';
    }
    
    protected function getScheduletime($pixel)
    {   
        date_default_timezone_set($pixel['time_zone']);
        $currentDate = date('Y-m-d');
        $currentTime = date('H:i');
        date_default_timezone_set(Config::settings('app_timezone'));
        
        return array(
            'startDate'     => $pixel['start_date'],
            'endDate'       => $pixel['end_date'],
            'startTime'     => $pixel['start_time'],
            'endTime'       => $pixel['end_time'],
            'currentDate'   => $currentDate,
            'currentTime'   => $currentTime
        );
    }

    protected function pixelTriggers($pixel)
    {
        if(Session::has(sprintf('extensions.delayedTransactions.steps.%d.main',$this->previousStepId)) && $pixel['pixel_type'] != 'General')
        {
            return false;
        }
        
        $isTestOrder = Session::get('steps.meta.isTestCard');
        if(!empty($isTestOrder) && !empty($pixel['fire_live_transactions']))
        {
            return false;
        }
        
        if(
            !empty($pixel['enable_custom_firing_schedule']))
        {
            $dateInfo = $this->getScheduletime($pixel);

            if(
                !empty($dateInfo['startDate']) && 
                !empty($dateInfo['endDate']) && 
                (
                    $dateInfo['startDate'] > $dateInfo['currentDate'] ||
                    $dateInfo['endDate'] < $dateInfo['currentDate']
                )
            ){
                return false;
            }
            
            if(
                !empty($dateInfo['startTime']) && 
                !empty($dateInfo['endTime']) && 
                (
                    $dateInfo['startTime'] > $dateInfo['currentTime'] ||
                    $dateInfo['endTime'] < $dateInfo['currentTime']
                )
            ){
                return false;
            }
        }
        
        if (Session::has(
                sprintf('steps.%d.pending', $this->previousStepId)
            )) 
        {
            return false;
        }
        
        if (Session::get(sprintf('steps.meta.skipPixelFire')) === true) {
            return false;
        }

        /// Added modification for cookie consent
        if($this->checkSkipPixelFire($pixel)) {
            return false;
        }
        
        $isValidDevice = $this->isValidDevice($pixel);
        
        if(
            $pixel['pixel_type'] == 'Submission' && 
            $isValidDevice && 
            !Session::get('steps.meta.isPrepaidFlow') && 
            !Session::get('steps.meta.isScrapFlow') 
            )
        {
            Session::set(
                sprintf(
                    'submissionPixels.pixel.%d.%s',
                    $pixel['id'], $pixel['pixel_placement']
                ), $pixel['html_pixel']
            );
        }

        if (
            !Session::has(
                sprintf('steps.%d.orderId', $this->previousStepId)
            )
                && 
                $pixel['pixel_type'] != 'General'
                && 
                $pixel['pixel_type'] != 'On Decline'
                &&
                !in_array($pixel['pixel_events'], $this->nonConversionFBEvents)
                &&
                !in_array($pixel['pixel_events_ga4'], $this->nonConversionGA4Events)
                &&
                !in_array($pixel['pixel_events_tik_tok'], $this->nonConversionTikTokEvents)
        ) {
            return false;
        }

        if (
            Session::get(
                sprintf(
                    'pixels.fireStatus.%d.%d',
                    $this->previousConfigId, $pixel['id']
                )
            ) === true
                && 
                $pixel['pixel_type'] != 'General'
                && 
                $pixel['pixel_type'] != 'On Decline'
                &&
                !in_array($pixel['pixel_events'], $this->nonConversionFBEvents)
                 &&
                !in_array($pixel['pixel_events_ga4'], $this->nonConversionGA4Events)
                &&
                !in_array($pixel['pixel_events_tik_tok'], $this->nonConversionTikTokEvents)
        ) {
            return false;
        }

        if (
            $pixel['prepaid'] === false &&
            (Session::get('steps.meta.isPrepaidFlow') === true ||  Session::get('steps.meta.isPrepaidFlowSkipFire') == true)
        ) {
            return false;
        }

        if (
            Session::get('steps.meta.isScrapFlow') === true  && 
            $pixel['pixel_type'] != 'General'
        ) {
            return false;
        }

        if (false === $this->isValidAffiliates($pixel)) {
            return false;
        }

        if (!empty($pixel['page']) && !empty($pixel['enable_page'])) {
            $pages = explode(',', $pixel['page']);
            $pages = array_map(function($v){
                return basename($v);
            }, $pages);
            
            $currentPageName = basename(
                Request::server()->get('SCRIPT_NAME')
            );
            if(!in_array($currentPageName, $pages))
            {
                return false;
            }
        }
        
        if (!empty($pixel['exceptions'])) {
            $excludepages = explode(',', $pixel['exceptions']);
            $excludepages = array_map(function($v){
                return basename($v);
            }, $excludepages);
            
            $currentPageName = basename(
                Request::server()->get('SCRIPT_NAME')
            );
            if(in_array($currentPageName, $excludepages))
            {
                return false;
            }
        }

        if ($isValidDevice && 
            $pixel['pixel_type'] != 'General' && 
            $pixel['pixel_type'] != 'On Decline' &&
            !in_array($pixel['pixel_events'], $this->nonConversionFBEvents) &&
            !in_array($pixel['pixel_events_ga4'], $this->nonConversionGA4Events) &&
            !in_array($pixel['pixel_events_tik_tok'], $this->nonConversionTikTokEvents)
         ) {
            Session::set(
                sprintf(
                    'pixels.fireStatus.%d.%d',
                    $this->previousConfigId, $pixel['id']
                ), true
            );
            return true;
        }
        
        if (
            Session::get(
                sprintf(
                    'generalPixels.fireStatus.%d',
                    $pixel['id']
                )
            ) === true
                && $pixel['pixel_type'] == 'General'
        ) {
            return false;
        }
        if (
            Session::get(
                sprintf(
                    'fbPixels.fireStatus.%d',
                    $pixel['id']
                )
            ) === true
                && 
                $pixel['pixel_type'] == 'FB Conversion Pixel (Server)' 
                && 
                in_array($pixel['pixel_events'], $this->nonConversionFBEvents)
        ) {

            return false;
        }
        
        if (
            Session::get(
                sprintf(
                    'GaFourPixels.fireStatus.%d',
                    $pixel['id']
                )
            ) === true
                && 
                $pixel['pixel_type'] == 'Google Analytics 4 (GA4)' 
                && 
                in_array($pixel['pixel_events_ga4'], $this->nonConversionGA4Events)
        ) {

            return false;
        }

        if (
            Session::get(
                sprintf(
                    'TikTokPixels.fireStatus.%d',
                    $pixel['id']
                )
            ) === true
                && 
                $pixel['pixel_type'] == 'Tik Tok Conversion Pixel (Server)' 
                && 
                in_array($pixel['pixel_events_tik_tok'], $this->nonConversionTikTokEvents)
        ) {

            return false;
        }
        
        if($pixel['pixel_type'] == 'General' && $isValidDevice)
        {
            $pixel['multi_fire'] = ($pixel['pixel_firing_option'] == "always") ? true : false;
            if(!empty($pixel['multi_fire'])){
                return true;
            }else{
                Session::set(
                    sprintf(
                        'generalPixels.fireStatus.%d',
                        $pixel['id']
                    ), true
                );
                return true;
            }
            
        }
        if(
            (
                $pixel['pixel_type'] == 'FB Conversion Pixel (Server)' && 
                in_array($pixel['pixel_events'], $this->nonConversionFBEvents)
            )  && 
            $isValidDevice
        )
        {
            Session::set(
                sprintf(
                    'fbPixels.fireStatus.%d',
                    $pixel['id']
                ), true
            );
            return true;
        }
        
        if(
            (
                $pixel['pixel_type'] == 'Tik Tok Conversion Pixel (Server)' && 
                in_array($pixel['pixel_events_tik_tok'], $this->nonConversionTikTokEvents)
            )  && 
            $isValidDevice
        )
        {
            Session::set(
                sprintf(
                    'TikTokPixels.fireStatus.%d',
                    $pixel['id']
                ), true
            );
            return true;
        }
        
        if(
            (
                $pixel['pixel_type'] == 'Google Analytics 4 (GA4)' && 
                in_array($pixel['pixel_events_ga4'], $this->nonConversionGA4Events)
            )  && 
            $isValidDevice
        )
        {
            Session::set(
                sprintf(
                    'GaFourPixels.fireStatus.%d',
                    $pixel['id']
                ), true
            );
            return true;
        }
        
        if($pixel['pixel_type'] == 'On Decline' && $isValidDevice)
        {
            Session::set(
                sprintf(
                    'declinePixels.pixel.%d.%s',
                    $pixel['id'], $pixel['pixel_placement']
                ), $pixel['html_pixel']
            );
           
            return false;
        }
        
        return false;
    }

    private function isValidAffiliates($pixel)
    {
        if(!empty($pixel['enable_affiliate_parameters']))
        {
            if (!empty($pixel['affiliate_id_value'])) {
                $affiliates = explode(',', $pixel['affiliate_id_value']);
                $affVal = Session::get(
                        sprintf('affiliates.%s', $pixel['affiliate_id_key'])
                    );
                if (
                    !in_array($affVal, $affiliates)
                ) {
                    return false;
                }
            }
            
            if (!empty($pixel['sub_id_value'])) {
                $subaffiliates = explode(',', $pixel['sub_id_value']);
                $affVal = Session::get(
                        sprintf('affiliates.%s', $pixel['sub_id_key'])
                    );
                if (
                    !in_array($affVal, $subaffiliates)
                ) {
                    return false;
                }
            }
        }

        return true;
    }

    private function isValidDevice($pixel)
    {

        if (!empty($pixel['device'])) {
            $devices = explode(',', $pixel['device']);
            if (!in_array('all', $devices)) {

                $detect = new MobileDetect();
                $ua  = '';

                if ($detect->isTablet()) {
                    $ua = 'tablet';
                    if($detect->version('iPad'))
                    {
                        $ua = 'ipad';
                    }
                }

                if (!$detect->isMobile() && !$detect->isTablet()) {
                    $ua = 'desktop';
                }

                if ($detect->isAndroidOS()) {
                    $ua = 'mobile_android';
                }

                if ($detect->isIphone()) {
                    $ua = 'iphone';
                }
                
                if (!in_array($ua, $devices)) {
                    return false;
                }
            }
        }

        return true;
    }

    public function encryptData($data, $algo = '')
    {
        if(empty($algo))
        {
            $algo = "sha256";
        }

        return hash($algo, $data);
    }

    protected function parseFBPostbackPixel($pixel)
    {
        $this->doFBPostBack($pixel);
        return '';
    }
    
    protected function parseGAFourPostbackPixel($pixel)
    {
        $this->doGaFourPostBack($pixel, false, $data = null);
        return '';
    }

    protected function parseTikTokPostbackPixel($pixel)
    {
        $this->doTikTokPostBack($pixel, false, $data = null);
        return '';
    }

    protected function doTikTokPostBack($data, $is_manual = false, $custom_data = array())
    {

        $payload      = $this->tiktokPayload; // required TikTok Payload 
        $campaignData = $this->getProductDetails();
        $pixelCode    = $data['tik_tok_pixel_id'];
        $externalID   = uniqid();

        $payload['pixel_code'] = $pixelCode;
        $payload['event']      = $data['pixel_events_tik_tok'];
        $payload['timestamp']  = date('c');
        $payload['event_id']   = $this->encryptData($externalID) . "_" . mt_rand(100000, 999999999);

        $email = Session::get('customer.email');
        $phone = Session::get('customer.phone');

        if (!empty($email) && !empty($phone)) {
            $payload['context']['user']['phone_number'] = $this->encryptData($phone);
            $payload['context']['user']['email'] = $this->encryptData($email);
            $payload['context']['user']['external_id'] = $this->encryptData($externalID);
        } else {
            unset($payload['context']['user']);
        }

        $payload['context']['page']['url'] = Request::getOfferUrl();
        $payload['context']['page']['referrer'] = Request::headers()->get('HTTP_REFERER');
        $payload['context']['user_agent'] = Request::headers()->get('HTTP_USER_AGENT');
        $payload['context']['ip'] = Request::getClientIp();

        if (
            $data['pixel_events_tik_tok'] == 'ViewContent' || 
            $data['pixel_events_tik_tok'] == 'PlaceAnOrder' || 
            $data['pixel_events_tik_tok'] == 'AddToCart' ||
            $data['pixel_events_tik_tok'] == 'CompletePayment'
        ) {
            foreach ($campaignData['products'] as $key => $value) {
                array_push($payload['properties']['contents'], array(
                    'content_id'     => (Session::get('crmType') === 'vrio') ? $value['vrioOfferId'] : $value['productId'],
                    'content_type'   => 'product',
                    'quantity'       => (Session::get('crmType') === 'vrio') ? $value['offerQuantity'] : $value['productQuantity'],
                    'price'          => (Session::get('crmType') === 'vrio') ? $value['offerPrice'] : $value['productPrice'],
                ));
            }
            $payload['properties']['currency'] = 'USD';
            $payload['properties']['value']    = $campaignData['orderTotal'];
        } else {
            unset($payload['properties']);
        }

        if (!empty($custom_data['custom_data'])) {
            array_merge($payload, $custom_data['custom_data']);
        }

        if (!empty(Session::get('queryParams.test_event_code'))) {
            $payload['test_event_code'] = Session::get('queryParams.test_event_code');
        } else {
            unset($payload['test_event_code']);
        }

        if (!empty(Session::get('queryParams.ttclid '))) { // get ttclid from URL if present
            $payload['context']['ad']['callback'] = Session::get('queryParams.ttclid');
        } else {
            unset($payload['context']['ad']);
        }

        $endPoint = self::TIK_TOK_API_ENDPOINT;
        $curlPostData = json_encode($payload);

        $headers = array(
            'Access-Token' => $data['tik_tok_access_token'],
            'Content-Type' => 'application/json'
        );

        $response = Http::post($endPoint, $curlPostData, $headers);

        if ($is_manual) {
            Session::set(
                sprintf(
                    'TikTokPixels.fireStatus.%d',
                    $data['id']
                ),
                true
            );
        }

        //if (DEV_MODE) {
        Session::set('TikTokPostBack.' . Session::get('steps.current.pageType') . '.' . $data['pixel_events_tik_tok'] . '.url', $endPoint);
        Session::set('TikTokPostBack.' . Session::get('steps.current.pageType') . '.' . $data['pixel_events_tik_tok'] . '.request', $curlPostData);
        Session::set('TikTokPostBack.' . Session::get('steps.current.pageType') . '.' . $data['pixel_events_tik_tok'] . '.response', $response);
        //}

        if (!empty($response['curlError'])) {
            return '';
        }
        return $response;
    }

    protected function doFBPostBack($data, $is_manual = false, $custom_data = array())
    {
        $curlPostData['data'] = array();
        $eventData['event_name'] = $data['pixel_events'];
        $eventData['event_time'] = time();
        $eventData['event_id']   = mt_rand(100000, 999999999);
        $eventData['event_source_url'] = Request::getOfferUrl();
        $eventData['action_source']    = 'website';

        $email = Session::get('customer.email');
        if(!empty($email))
        {
            $eventData['user_data']['em'] = $this->encryptData($email);
        }

        // Adding modifications for v18.0
        if(!empty($data['pixel_versions']) && $data['pixel_versions'] == "v18.0"){
            $str = bin2hex(openssl_random_pseudo_bytes(32));
            $str = htmlspecialchars($str, ENT_QUOTES | ENT_HTML401, 'UTF-8');
            $str = substr($str,0,10);
            $id = rand(0,1000).rand();
            $eventData['event_id'] = $id.$str.'2';


            if($data['pixel_events'] == 'Purchase')
            {
                $filtered = array_filter(
                    ['em'=>Session::get('customer.email'),
                     'fn'=>Session::get('customer.firstName'),
                     'ln'=>Session::get('customer.lastName'),
                     'ph'=>Session::get('customer.phone'),
                     'country'=>Session::get('customer.shippingCountry'),
                     'ct'=>Session::get('customer.shippingCity'),
                     'st'=>Session::get('customer.shippingState'),
                     'zp'=>Session::get('customer.shippingZip'),
                    ],
                    function ($val, $key) {
                        return !empty($val) ? true : false;
                    },
                    ARRAY_FILTER_USE_BOTH
                ); 

                array_walk($filtered, function($val, $key) use(&$result) {
                    $result['user_data'][$key] = $this->encryptData($val);
                });
                $eventData = array_merge($eventData,$result);

            }
            else{
                unset($eventData['user_data']['em']);
            }
        }

        // Adding modifications for v18.0 ends here

        if(in_array($data['pixel_events'], $this->nonConversionFBEvents))
        {
            $eventData['custom_data']['page'] = Session::get('steps.' . $this->currentStepId . '.pageType');
        }

        $eventData['user_data']['client_ip_address'] = Request::getClientIp();
        $eventData['user_data']['client_user_agent'] = Request::headers()->get('HTTP_USER_AGENT');

        if($data['pixel_events'] == 'Purchase')
        {
            $eventData['custom_data']['value'] = $this->getOrderTotal();
            $eventData['custom_data']['currency'] = 'USD';
            $eventData['custom_data']['content_type'] = 'product';
            $eventData['custom_data']['transaction_id'] = Session::get(
                sprintf('steps.%d.orderId', $this->previousStepId)
            );
            $eventData['opt_out'] = false;
        }
        
        if(!empty($custom_data['user_data'])){
            $eventData['user_data'] = array_merge($eventData['user_data'], $custom_data['user_data']);
            unset($custom_data['user_data']);
        }

        if(!empty($custom_data))
        {
            if (!empty($eventData['custom_data'])) {
                $eventData['custom_data'] = array_merge($eventData['custom_data'], $custom_data);
            } else { 
                $eventData['custom_data'] = $custom_data;
            }
        }
        
        array_push($curlPostData['data'], $eventData);

        if(!empty(Session::get('queryParams.test_event_code')))
        {
            $curlPostData['test_event_code'] = Session::get('queryParams.test_event_code');
        }

        $ver = "v9.0";
        if(!empty($data['pixel_versions']))
        {
            $ver = $data['pixel_versions'];
        }
        
        $endPoint = "https://graph.facebook.com/".$ver."/".$data['fb_pixel_id']."/events?access_token=".$data['fb_access_token'];
        $curlPostData = json_encode($curlPostData);
        
        $headers = array('Content-Type' => 'application/json');
        $response = Http::post(
            $endPoint, $curlPostData, $headers
        );

        if($is_manual)
        {
            Session::set(
                sprintf(
                    'fbPixels.fireStatus.%d',
                    $data['id']
                ), true
            );
        }

        //if (DEV_MODE) {
        Session::set('fbpostBack.' . Session::get('steps.current.pageType') . '.' . $data['pixel_events'] . '.url', $endPoint);
        Session::set('fbpostBack.' . Session::get('steps.current.pageType') . '.' . $data['pixel_events'] . '.request', $curlPostData);
        Session::set('fbpostBack.' . Session::get('steps.current.pageType') . '.' . $data['pixel_events'] . '.response', $response);
        //}
        
        $httpCode = isset($response['httpCode'])? $response['httpCode']: 0;
        if(!empty($response['curlError']) || $httpCode >= 400)
        {
            try {
                $this->createDatabase(); 
                $connection = $this->getDatabaseConnection();
                $connection->table($this->tableName)->insert(array(
                    'pixel_id'      => $data['id'],
                    'endpoint'      => $endPoint,
                    'http_request'  => $curlPostData,
                    'http_response' => json_encode($response),
                    'status'        => 0,            
                    'created_at'    => date('Y-m-d H:i:s', time()),
                    'updated_at'    => date('Y-m-d H:i:s', time()),
                    'scheduled_at'  => date('Y-m-d H:i:s', strtotime('now +1 hour'))
                ));
            } catch (Exception $ex) {
            }

        }
        
        if (!empty($response['curlError'])) {
            return '';
        }
        return $response;
    }

    /// Added modifications for v18.0

    public function checkSkipPixelFire($pixel){
        return ((
            (!empty(Session::get('skipConversionPixelFire')) && Session::get('skipConversionPixelFire') === true) ||
            (!empty($pixel['skipConversionPixelFire']) && $pixel['skipConversionPixelFire'] == true)
        ) &&
        !empty(
            Config::extensionsConfig('CookieConsent.skip_pixel_fire') && 
            Config::extensionsConfig('CookieConsent.skip_pixel_fire')
        )) && in_array($pixel['pixel_events'],$this->conversionPixelEvents) ? true : false;       
    }
    
    public function createDatabase()
    {
        try {
            $sql = 'CREATE TABLE IF NOT EXISTS `fbpixelfallback` (
                                `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
                                `pixel_id` INTEGER NOT NULL,
                                `endpoint` TEXT NOT NULL,
                                `http_request` TEXT NOT NULL,
                                `http_response` TEXT NOT NULL,
                                `status`  INTEGER NOT NULL,
                                `created_at` TEXT NOT NULL,
                                `updated_at` TEXT NOT NULL,
                                `scheduled_at` TEXT NOT NULL
                                )';

            $this->getDatabaseConnection()->query($sql);
        } catch (Exception $ex) {
        }
    }

    public function createDatabase1()
    {
        try {
            $sql = 'CREATE TABLE IF NOT EXISTS `postbackpixel` (
                                `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
                                `pixel_id` INTEGER NOT NULL,
                                `pixel_type` TEXT NOT NULL,
                                `data_process_type` TEXT NOT NULL,
                                `process_days` TEXT NOT NULL,
                                `postback_url` TEXT NOT NULL,
                                `http_response` TEXT NOT NULL,
                                `status`  INTEGER NOT NULL,
                                `created_at` TEXT NOT NULL,
                                `updated_at` TEXT NOT NULL,
                                `scheduled_at` TEXT NOT NULL
                                )';
            $this->getDatabaseConnection1()->query($sql);
        } catch (Exception $ex) {
        }
    }



    private function getDatabaseConnection()
    {
        try {
            $this->fbPixelFilePath = STORAGE_DIR . DS . 'fbpixel.sqlite';

            if (!file_exists($this->fbPixelFilePath)) {
                file_put_contents($this->fbPixelFilePath, '');
            }

            if (!is_writable($this->fbPixelFilePath)) {
                return;
            }

            $factory = new ConnectionFactory();
            return $factory->make(array(
                'driver' => 'sqlite',
                'database' => STORAGE_DIR . DS . 'fbpixel.sqlite',
            ));
        } catch (Exception $ex) {
        }
    }

    private function getDatabaseConnection1()
    {
        try {
            $this->postbackPixelFilePath = STORAGE_DIR . DS . 'postbackpixel.sqlite';
            if (!file_exists($this->postbackPixelFilePath)) {
                file_put_contents($this->postbackPixelFilePath, '');
            }
            if (!is_writable($this->postbackPixelFilePath)) {
                return;
            }
            $factory = new ConnectionFactory();
            return $factory->make(array(
                'driver' => 'sqlite',
                'database' => STORAGE_DIR . DS . 'postbackpixel.sqlite',
            ));
        } catch (Exception $ex) {
        }
    }


    public function fireManualFBPixels($criteria)
    {
        $pixels  = Config::pixels();

        foreach ($pixels as $pixel) {
            if ($this->isValidAffiliates($pixel) === false) {
                continue;
            }
            if (empty($pixel['fb_manual_trigger'])) {

                continue;
            }

            /// Added modification for cookie consent
            if($this->checkSkipPixelFire($pixel)) {
                return false;
            }
            
            if (!empty($pixel['pixel_events']) && $pixel['pixel_events'] == 'CustomEvent') 
            {
                $pixel['pixel_events'] = $criteria['event_name'];
            }
            elseif(
                    !empty($pixel['pixel_events']) && 
                    $pixel['pixel_events'] != $criteria['event_name']
                )
            {
               continue;
            }

            if (!empty($pixel['page']) && !empty($pixel['enable_page'])) {
                $pages = explode(',', $pixel['page']);
                $pages = array_map(function($v){
                    return basename($v);
                }, $pages);

                $currentPageName = $criteria['page_name'];
                
                if(!in_array($currentPageName, $pages))
                {
                    continue;
                }
            }

            if (!empty($pixel['fb_manual_trigger'])) {
                array_push($this->manualPixels, $pixel);
            }
        }

        if(!empty($this->manualPixels)){
            foreach ($this->manualPixels as $pixel) {
               return $this->doFBPostBack($pixel, true, $criteria['custom_data']);
            }
        }
    }

    public function fireManualGAFourPixel($data)
    {
        $pixels = Config::pixels();
        $ga4    = array();

        foreach ($pixels as $pixel) {

            if ($this->isValidAffiliates($pixel) === false) {
                continue;
            }

            if (empty($pixel['ga4_manual_trigger'])) {
                continue;
            }

            if (!empty($pixel['pixel_events_ga4']) && $pixel['pixel_events_ga4'] != $data['event_name']) {
                continue;
            }

            if (!empty($pixel['page']) && !empty($pixel['enable_page'])) {
                $pages = explode(',', $pixel['page']);
                $pages = array_map(function ($v) {
                    return basename($v);
                }, $pages);

                $currentPageName = $data['page_name'];

                if (!in_array($currentPageName, $pages)) {
                    continue;
                }
            }

            if ($pixel['pixel_type'] === 'Google Analytics 4 (GA4)' && !empty($pixel['ga4_manual_trigger'])) {
                array_push($ga4, $pixel);
            }
        }

        if (!empty($ga4)) {
            foreach ($ga4 as $pixel) {
                return $this->doGaFourPostBack($pixel, true, $data);
            }
        }
    }

    public function fireManualTikTokPixel($data)
    {
        $pixels = Config::pixels();
        $tiktokPixel    = array();

        foreach ($pixels as $pixel) {

            if ($this->isValidAffiliates($pixel) === false) {
                continue;
            }

            if (empty($pixel['tik_tok_manual_trigger'])) {
                continue;
            }

            /// Added modification for cookie consent
            if($this->checkSkipPixelFire($pixel)) {
                return false;
            }

            if (!empty($pixel['pixel_events_tik_tok']) && $pixel['pixel_events_tik_tok'] != $data['event_name']) {
                continue;
            }

            if (!empty($pixel['page']) && !empty($pixel['enable_page'])) {
                $pages = explode(',', $pixel['page']);
                $pages = array_map(function ($v) {
                    return basename($v);
                }, $pages);

                $currentPageName = $data['page_name'];

                if (!in_array($currentPageName, $pages)) {
                    continue;
                }
            }

            if ($pixel['pixel_type'] === 'Tik Tok Conversion Pixel (Server)' && !empty($pixel['tik_tok_manual_trigger'])) {
                array_push($tiktokPixel, $pixel);
            }
        }

        if (!empty($tiktokPixel)) {
            foreach ($tiktokPixel as $pixel) {
                return $this->doTikTokPostBack($pixel, true, $data);
            }
        }
    }

    public function stoc($snakecase, $class = false)
    {
        $parts     = explode('_', $snakecase);
        $camelcase = implode('', array_map('ucfirst', $parts));
        if ($class) {
            return $camelcase;
        }
        return lcfirst($camelcase);
    }

    public function doGaFourPostBack($ga4, $is_manual = false, $data)
    {
        $eventToBeFired = null;
        $eventToBeFired = $ga4['pixel_events_ga4'];
        if (!empty($eventToBeFired)) {
            $eventToBeFired = $this->stoc($eventToBeFired);
            $this->$eventToBeFired($ga4, $data);
        }

        if ($is_manual) {
            Session::set(
                sprintf(
                    'GaFourPixels.fireStatus.%d',
                    $ga4['id']
                ),
                true
            );
        }
    }

    public function viewCart($ga4, $data)
    {
        $campaignData = $this->getProductDetails();        
        $payLoad = $campaignData['items'];

        $payLoad['events'][0]['name']               = $ga4['pixel_events_ga4'];
        $payLoad['events'][0]['params']['currency'] = 'USD';
        $payLoad['events'][0]['params']['value']    = $campaignData['orderTotal'];
        unset($payLoad['events'][0]['params']['transaction_id']);

        if (!empty($data['custom_data'])) {
            $requiredPayload = array_merge($payLoad['events'][0]['params'], $data['custom_data']);
            $payLoad['events'][0]['params'] = $requiredPayload;
        }

        $this->ga4CurlPost($payLoad, $ga4);
    }

    public function generateLead($ga4, $data)
    {
        $campaignData = $this->getProductDetails();        
        $payLoad = $campaignData['items'];

        $payLoad['events'][0]['name']               = $ga4['pixel_events_ga4'];
        $payLoad['events'][0]['params']['currency'] = 'USD';
        $payLoad['events'][0]['params']['value']    = $campaignData['orderTotal'];

        unset($payLoad['events'][0]['params']['transaction_id'], $payLoad['events'][0]['params']['items']);

        if (!empty($data['custom_data'])) {
            $requiredPayload = array_merge($payLoad['events'][0]['params'], $data['custom_data']);
            $payLoad['events'][0]['params'] = $requiredPayload;
        }

        $this->ga4CurlPost($payLoad, $ga4);
    }

    public function addToCart($ga4, $data)
    {
        $campaignData = $this->getProductDetails();        
        $payLoad = $campaignData['items'];

        $payLoad['events'][0]['name']               = $ga4['pixel_events_ga4'];
        $payLoad['events'][0]['params']['currency'] = 'USD';
        $payLoad['events'][0]['params']['value']    = $campaignData['orderTotal'];
        unset($payLoad['events'][0]['params']['transaction_id']);

        if (!empty($data['custom_data'])) {
            $requiredPayload = array_merge($payLoad['events'][0]['params'], $data['custom_data']);
            $payLoad['events'][0]['params'] = $requiredPayload;
        }

        $this->ga4CurlPost($payLoad, $ga4);
    }

    public function removeFromCart($ga4, $data)
    {
        $campaignData = $this->getProductDetails();        
        $payLoad = $campaignData['items'];

        $payLoad['events'][0]['name']               = $ga4['pixel_events_ga4'];
        $payLoad['events'][0]['params']['currency'] = 'USD';
        $payLoad['events'][0]['params']['value']    = $campaignData['orderTotal'];
        unset($payLoad['events'][0]['params']['transaction_id']);

        if (!empty($data['custom_data'])) {
            $requiredPayload = array_merge($payLoad['events'][0]['params'], $data['custom_data']);
            $payLoad['events'][0]['params'] = $requiredPayload;
        }

        $this->ga4CurlPost($payLoad, $ga4);
    }

    public function beginCheckout($ga4, $data)
    {
        $campaignData = $this->getProductDetails();        
        $payLoad = $campaignData['items'];

        $payLoad['events'][0]['name']               = $ga4['pixel_events_ga4'];
        $payLoad['events'][0]['params']['currency'] = 'USD';
        $payLoad['events'][0]['params']['value']    = $campaignData['orderTotal'];
        unset($payLoad['events'][0]['params']['transaction_id']);

        if (!empty($data['custom_data'])) {
            $requiredPayload = array_merge($payLoad['events'][0]['params'], $data['custom_data']);
            $payLoad['events'][0]['params'] = $requiredPayload;
        }

        $this->ga4CurlPost($payLoad, $ga4);
    }

    public function purchase($ga4, $data)
    {
        $campaignData = $this->getProductDetails();        
        $payLoad = $campaignData['items'];

        $payLoad['events'][0]['name']                     = $ga4['pixel_events_ga4'];
        $payLoad['events'][0]['params']['currency']       = 'USD';
        $payLoad['events'][0]['params']['value']          = $campaignData['orderTotal'];
        $payLoad['events'][0]['params']['transaction_id'] = Session::get(sprintf('steps.%d.orderId', $this->previousStepId)); 
        $payLoad['events'][0]['params']['shipping']       = $ga4['shipping'];
        $payLoad['events'][0]['params']['tax']            = $ga4['tax'];
        
        if (!empty($data['custom_data'])) {
            $requiredPayload = array_merge($payLoad['events'][0]['params'], $data['custom_data']);
            $payLoad['events'][0]['params'] = $requiredPayload;
        }

        $this->ga4CurlPost($payLoad, $ga4);
    }
    
    public function addPaymentInfo($ga4, $data)
    {
        $campaignData = $this->getProductDetails();        
        $payLoad = $campaignData['items'];
        
        $payLoad['events'][0]['name']                     = $ga4['pixel_events_ga4'];
        $payLoad['events'][0]['params']['currency']       = 'USD';
        $payLoad['events'][0]['params']['value']          = $campaignData['orderTotal'];
        unset($payLoad['events'][0]['params']['transaction_id']);

        if (!empty($data['custom_data'])) {
            $requiredPayload = array_merge($payLoad['events'][0]['params'], $data['custom_data']);
            $payLoad['events'][0]['params'] = $requiredPayload;
        }
        $this->ga4CurlPost($payLoad, $ga4);
    }

    public function getProductDetails()
    {
        $campaignData = array();
        $orderTotal = 0.00;
        $products = Session::get(
            sprintf('steps.%d.products', $this->previousStepId)
        );
        $campaignLabel = '';

        if(empty($products)){
            $configuration = new Configuration($this->previousConfigId);
            $campaignId = $configuration->getCampaignIds();
            $data = Campaign::find($campaignId[0]);
            $campaignLabel = $data['campaign_lable'];
            $products = $data['product_array'];
        }
        
        if (!empty($products)) {
            foreach ($products as $value) {
                if (Session::get('crmType') === 'vrio') {
                    $orderTotal += ($value['offerPrice'] * $value['offerQuantity']) + $campaignData['shippingPrice'];
                } else {
                    $orderTotal += ($value['productPrice'] * $value['productQuantity']) + $campaignData['shippingPrice'];
                }
            }
            $campaignData['products']   = $products;
            $campaignData['orderTotal'] = number_format($orderTotal , 2);
            
            $payLoad = $this->gaFourPayLoad;

            foreach ($campaignData['products'] as $key => $value) {
                $value['productPrice'] = !empty($value['productPrice']) ? $value['productPrice'] : '0.00';
                $value['offerPrice'] = !empty($value['offerPrice']) ? $value['offerPrice'] : '0.00';
                array_push($payLoad['events'][0]['params']['items'], array(
                    'item_id'        => (Session::get('crmType') === 'vrio') ? $value['vrioOfferId'] : $value['productId'],
                    'item_name'      => !empty($value['campaign_lable']) ? $value['campaign_lable'] : $campaignLabel,
                    'quantity'       => (Session::get('crmType') === 'vrio') ? $value['offerQuantity'] : $value['productQuantity'],
                    'price'          => (Session::get('crmType') === 'vrio') ? $value['offerPrice'] : $value['productPrice'],
                    'currency'       => 'USD',
                ));
            }
            $campaignData['items'] = $payLoad;
        }
        return $campaignData;
    }

    public function ga4CurlPost($payLoad, $ga4)
    {
        if(!empty($_COOKIE['_ga']))
        {
            $clientId = preg_replace("/^.+\.(.+?\..+?)$/", "\\1", @$_COOKIE['_ga']);
        }
        else{
            $clientId = !empty(Session::get('ga4.client_id')) ? Session::get('ga4.client_id') : '';
        }
        $measurementId = $ga4['ga4_measurement_id'];
        $apiSecret     = $ga4['ga4_secret_key'];

        $payLoad['client_id'] = $clientId;
        $payLoad  = json_encode($payLoad);
        
        $endPoint = self::API_ENDPOINT . $measurementId . '&api_secret=' . $apiSecret;
        $headers  = array('Content-Type' => 'application/json');
        $response = Http::post($endPoint, $payLoad, $headers);

        Session::set(sprintf('GaFourPixels.fireStatus.%d', $ga4['id']), true);
        Session::set('GaFourPostBack.' . Session::get('steps.current.pageType') . '.' . $ga4['pixel_events_ga4'] . '.endPoint', $endPoint);
        Session::set('GaFourPostBack.' . Session::get('steps.current.pageType') . '.' . $ga4['pixel_events_ga4'] . '.payLoad', $payLoad);
        Session::set('GaFourPostBack.' . Session::get('steps.current.pageType') . '.' . $ga4['pixel_events_ga4'] . '.response', $response);
        Session::set('GaFourPostBack.' . Session::get('steps.current.pageType') . '.' . $ga4['pixel_events_ga4'] . '.httpCode', Http::getHttpResponseCode());
    }
    
}