<?php

namespace Application\Model;

use Application\CrmPayload;
use Application\CrmResponse;
use Application\Http;
use Application\Response;
use Application\Request;
use Exception;

class Twentyninenext extends BaseCrm
{

    private $methodSlugMapping = array(
        'prospect' => 'users',
        'newOrderWithProspect' => 'orders',
        'newOrderCardOnFile' => 'add-line-items',
        'newOrderWithCardToken'  => 'orders',
        'upsell' => 'add-line-items',
        'newOrder' => 'orders',
        'newCart'  => 'carts',
        'cartUpdate' => 'carts/',
        'addSubscription' => 'subscriptions'
    );

    private $memberShipMethods = array(
        'orderView'  => 'orders',
    );
    
    private $currentSlug;    
    private $enableCart  = false;
    private $cartDetails = false;
    private $subscriptionData = false;
    private $cartId = 0;

    public function __construct($crmId)
    {
        parent::__construct($crmId);
    }

    protected function beforeAnyCrmClassMethodCall()
    {
        $this->params = $this->response = array();
    }

    protected function prospect()
    {
        $this->params = array_replace($this->params, CrmPayload::all());
        
        $this->prepareBasicDetails(true);
        $this->prepareShippingDetails(true);
        $this->prepareAffiliates();

        if (!empty($this->params['meta.enableCart']) && !empty($this->params['meta.abondonedCartDetails'])) {
            $this->enableCart  = true;
            $this->cartDetails = $this->params['meta.abondonedCartDetails'];
        }

        $this->params['slug'] = $this->methodSlugMapping['prospect'];
        
        if ($this->makeHttpRequest() === false)
        {
            return false;
        }

        $oldResponse = $this->response;
        $oldCrmPayload = CrmPayload::all();

        if (!empty($this->enableCart)) {
            $this->newCart();
        }

        if (!empty($this->response['status']) && !empty($this->response['lines'])) {
            $this->cartId = $this->response['id'];
        }

        $this->response = $oldResponse;
        CrmPayload::replace($oldCrmPayload);

        if (!empty($this->cartId)) {
            CrmPayload::set('meta.cartId', $this->cartId);
        }

        CrmResponse::replace(array(
            'success' => true,
            'prospectId' => $this->response['id'],
        ));
    }
    
    public function addSubscription($subscription)
    {
        $this->params = array_replace($this->params, CrmPayload::all());
        $this->params['card_token'] = !empty(CrmPayload::get('card_token')) ? CrmPayload::get('card_token') : CrmPayload::get('meta.card_token');
        $this->prepareProductDetails($subscription);
        $this->prepareShippingDetails();
        $this->prepareBillingDetails();
        $this->params['cardType'] = 'external';
        $this->prepareCardDetails();
        if (!empty(CrmPayload::get('enable_affiliates'))) {
            $this->prepareAffiliates();
        }

        $this->params['interval']          = $this->subscriptionData['interval'];
        $this->params['interval_count']    = $this->subscriptionData['interval_count'];
        $this->params['next_renewal_date'] = $this->subscriptionData['next_renewal_date'];

        if(!empty($subscription['products'][0]['shippingPrice'])) {
            $this->params['shipping_price'] = $subscription['products'][0]['shippingPrice'];
        }

        $this->params['slug'] = $this->methodSlugMapping['addSubscription'];
        unset($this->params['email'], $this->params['send_receipt'], $this->params['use_default_shipping_address'], $this->params['use_default_billing_address'], $this->params['customerId'], $this->params['couponCode'], $this->params['stockrecord_id'], $this->params['forceGatewayId'], $this->params['forceGatewayId'], $this->params['previousOrderId'], $this->params['card_token'], $this->params['previousOrderId'], $this->params['domain'], $this->params['device'], $this->params['vouchers'],$this->params['remove_user_details']);

        $this->cleanParamsBeforePost();
        $this->makeHttpRequest();
    }
    
    public function newCart()
    {
        if (empty($this->cartDetails) || empty($this->response['id'])) {
            return false;
        } 
       
        $this->params['lines'][0] = $this->cartDetails; 
        $this->params['user']['id'] = $this->response['id'];
        $this->params['slug'] = $this->methodSlugMapping['newCart'];
        
        if (!empty(CrmPayload::get('meta.couponCode'))) {
            $this->params['couponCode'] = $this->params['meta.couponCode'];
        }

        if ($this->makeHttpRequest() === false) {
            return false;
        }
    }

    protected function newOrderWithProspect()
    {
        $this->params = array_replace($this->params, CrmPayload::all());
        $this->prepareShippingDetails();
        $this->prepareProductDetails();

        $this->prepareAffiliates();
        $this->prepareCardDetails();
        $this->prepareBillingDetails();

        if (!empty($this->params['meta.cartId'])) {
            $this->cartId = $this->params['meta.cartId'];
            $this->updateCart();
        }

        $this->params['user']['id'] = $this->params['prospectId'];
        $this->params['slug']       = $this->methodSlugMapping['newOrderWithProspect'];

        $this->cleanParamsBeforePost();

        if ($this->makeHttpRequest() === false)
        {
            return false;
        }

        $oldResponse   = $this->response;
        $oldCrmPayload = CrmPayload::all();

        $enableSubscription = CrmPayload::get('meta.enableSubscription');
        if (!empty($enableSubscription)) {
            $this->subscriptionData = CrmPayload::get('meta.subscriptionData');
        }

        if (!empty($this->subscriptionData)) {
            $cardToken = $this->response['transactions'][0]['payment_details']['card_token'];
            CrmPayload::set('meta.card_token', $cardToken);
            $this->addSubscription($this->subscriptionData);
        }

        $this->response = $oldResponse;
        CrmPayload::replace($oldCrmPayload);
        
        CrmResponse::replace(array(
            'success'        => true,
            'orderId'        => $this->response['number'],
            'customerId'     => $this->response['user']['id'],
            'transactions'   => !empty($this->response['transactions']) ? $this->response['transactions'] : '',
            'total_excl_tax' => !empty($this->response['total_excl_tax']) ? $this->response['total_excl_tax'] : '',
            'total_incl_tax' => !empty($this->response['total_incl_tax']) ? $this->response['total_incl_tax'] : '',
            'card_token'     => !empty($this->response['transactions'][0]['payment_details']['card_token']) ? $this->response['transactions'][0]['payment_details']['card_token'] : '',
        ));
    }

    public function updateCart()
    {
        $this->params['user']['id'] = $this->params['prospectId'];
        $this->params['slug'] = $this->methodSlugMapping['cartUpdate'].$this->cartId;

        if ($this->makeHttpRequest() === false)
        {
            return false;
        }
    }

    protected function newOrder()
    {
        $this->params = array_replace($this->params, CrmPayload::all());

        $this->prepareBasicDetails();
        $this->prepareShippingDetails();
        $this->prepareBillingDetails();
        CrmPayload::set('meta.allow_shipping_id', true);
        $this->prepareProductDetails();

        $this->prepareAffiliates();
        $this->prepareCardDetails();

        $this->params['slug'] = $this->methodSlugMapping['newOrder'];

        $this->cleanParamsBeforePost();
        if (($this->params['payment_method'] == 'paypal' || $this->params['payment_method'] == 'klarna') && !empty($this->params['remove_user_details']))
        {
            unset($this->params['remove_user_details'], $this->params['user'], $this->params['shipping_address'],
                $this->params['billing_same_as_shipping_address']);
        }

        if ($this->makeHttpRequest() === false)
        {
            return false;
        }

        $oldResponse   = $this->response;
        $oldCrmPayload = CrmPayload::all(); 
        
        $enableSubscription = CrmPayload::get('meta.enableSubscription');
        if (!empty($enableSubscription)) {
            $this->subscriptionData = CrmPayload::get('meta.subscriptionData');
        }

        if (!empty($this->subscriptionData)) {
            $cardToken = $this->response['transactions'][0]['payment_details']['card_token'];
            CrmPayload::set('meta.card_token', $cardToken);
            $this->addSubscription($this->subscriptionData);
        }

        $this->response = $oldResponse;
        CrmPayload::replace($oldCrmPayload);

        CrmResponse::replace(array(
            'success'        => true,
            'orderId'        => $this->response['number'],
            'customerId'     => $this->response['user']['id'],
            'transactions'   => !empty($this->response['transactions']) ? $this->response['transactions'] : '',
            'total_excl_tax' => !empty($this->response['total_excl_tax']) ? $this->response['total_excl_tax'] : '',
            'total_incl_tax' => !empty($this->response['total_incl_tax']) ? $this->response['total_incl_tax'] : '',
            'card_token'     => !empty($this->response['transactions'][0]['payment_details']['card_token']) ? $this->response['transactions'][0]['payment_details']['card_token'] : '',
        ));
    }

    protected function newOrderCardOnFile()
    {
        return $this->upsell();
    }

    protected function newOrderWithCardToken() {

        $this->params = array_replace($this->params, CrmPayload::all());

        $this->prepareAffiliates();
        CrmPayload::set('meta.allow_shipping_id', true);
        $this->prepareProductDetails();
        $this->params['cardType'] = 'external';
        $this->prepareCardDetails();

        $this->params['slug'] = $this->methodSlugMapping['newOrderWithCardToken'];


        if (!empty(CrmPayload::get('customerId'))) {
            $this->params['user']['id'] = CrmPayload::get('customerId');
        }

        if (!empty(CrmPayload::get('send_receipt'))) {
            $this->params['send_receipt'] = CrmPayload::get('send_receipt');
        }

        if (!empty(CrmPayload::get('use_default_shipping_address'))) {
            $this->params['use_default_shipping_address'] = CrmPayload::get('use_default_shipping_address');
        }

        if (!empty(CrmPayload::get('use_default_billing_address'))) {
            $this->params['use_default_billing_address'] = CrmPayload::get('use_default_billing_address');
        }

        if (!empty(CrmPayload::get('stockrecord_id'))) {
            unset($this->params['stockrecord_id']);
            $this->params['stockrecord_id'] = 'true';
        }

        if (!empty(CrmPayload::get('couponCode'))) {
            $this->params['couponCode'] = CrmPayload::get('couponCode');
        }


        unset($this->params['productDetails'], $this->params['email'], $this->params['phone'], $this->params['shippingAddress1'], $this->params['shippingCity'], $this->params['shippingZip'],
            $this->params['shippingState'], $this->params['shippingCountry'], $this->params['previousOrderId']
            , $this->params['customerId'], $this->params['card_token'], $this->params['shippingAddress2'],  $this->params['billingAddress2'], $this->params['newSubscription']);


        $this->cleanParamsBeforePost();

        if ($this->makeHttpRequest() === false) {
            return false;
        }

        $oldResponse   = $this->response;
        $oldCrmPayload = CrmPayload::all();

        $enableSubscription = CrmPayload::get('meta.enableSubscription');
        if (!empty($enableSubscription)) {
            $this->subscriptionData = CrmPayload::get('meta.subscriptionData');
        }

        if (!empty($this->subscriptionData)) {
            if (!empty($oldResponse['user']['id'])) {
                $this->params['user']['id'] = $oldResponse['user']['id'];
            }
            $this->addSubscription($this->subscriptionData);
        }

        $this->response = $oldResponse;
        CrmPayload::replace($oldCrmPayload);

        CrmResponse::replace(array(
            'success'        => true,
            'orderId'        => $this->response['number'],
            'customerId'     => $this->response['user']['id'],
            'transactions'   => !empty($this->response['transactions']) ? $this->response['transactions'] : '',
            'total_excl_tax' => !empty($this->response['total_excl_tax']) ? $this->response['total_excl_tax'] : '',
            'total_incl_tax' => !empty($this->response['total_incl_tax']) ? $this->response['total_incl_tax'] : '',
        ));

    }

    protected function upsell()
    {
        $this->params['products'] = array_replace($this->params, CrmPayload::get('products'));
        $this->prepareProductDetails();
        $this->params['slug'] = 'orders/' . CrmPayload::get('previousOrderId') . '/' . $this->methodSlugMapping['upsell'];

        unset($this->params['productDetails']);
        $this->cleanParamsBeforePost();

        if ($this->makeHttpRequest() === false)
        {
            return false;
        }

        $oldResponse   = $this->response;
        $oldCrmPayload = CrmPayload::all();

        $enableSubscription = CrmPayload::get('meta.enableSubscription');
        if (!empty($enableSubscription)) {
            $this->subscriptionData = CrmPayload::get('meta.subscriptionData');
        }
        if (!empty($this->subscriptionData)) {
            if (!empty($oldResponse['user']['id'])) {
                $this->params['user']['id'] = $oldResponse['user']['id'];
            }
            $this->addSubscription($this->subscriptionData);
        }

        $this->response = $oldResponse;
        CrmPayload::replace($oldCrmPayload);

        CrmResponse::replace(array(
            'success'        => true,
            'orderId'        => $this->response['number'],
            'customerId'     => $this->response['user']['id'],
            'transactions'   => !empty($this->response['transactions']) ? $this->response['transactions'] : '',
            'total_excl_tax' => !empty($this->response['total_excl_tax']) ? $this->response['total_excl_tax'] : '',
            'total_incl_tax' => !empty($this->response['total_incl_tax']) ? $this->response['total_incl_tax'] : '',
        ));
    }

    private function beforeHttpRequest()
    {

        if (!empty($this->params['couponCode'])) {
            $this->params['vouchers'] = explode(',', $this->params['couponCode']);
        }

        unset(
                $this->params['forceGatewayId'], $this->params['preserveGateway'], $this->params['parentOrderId'], $this->params['userIsAt'], $this->params['userAgent'], $this->params['products'], $this->params['couponCode'], $this->params['phone_prefix']
        );

        $metaKeys = preg_grep('/^meta\..+$/', array_keys($this->params));
        foreach ($metaKeys as $metaKey)
        {
            unset($this->params[$metaKey]);
        }

        $this->response = array();
    }

    private function makeHttpRequest()
    {
        $this->beforeHttpRequest();
        $url = $this->getUrl($this->params['slug']);
        $this->currentSlug = $this->params['slug'];
        unset($this->params['slug']);

        $params = array_filter($this->params);
        
        if ($this->currentSlug == 'carts' || $this->currentSlug == 'carts/'.$this->cartId) {
            unset($params['prospectId'], $params['shipping_code'], $params['first_name'],  $params['last_name'], $params['ip'], $params['phone_number'], $params['firstName'], $params['lastName'], $params['userIsAt'], $params['billing_same_as_shipping_address'], $params['cardType'], $params['billingSameAsShipping'], $params['cardNumber'], $params['email'], $params['shipping_address'], $params['payment_method'], $params['payment_details'], $params['addresses']);
        }
        
        $headers = array('Content-Type' => 'application/json', 'Authorization' => $this->token . $this->username);
        if ($this->currentSlug == 'carts/'.$this->cartId) {
            $response = Http::customRequest($url, json_encode($params), $headers, $method = 'PUT');
        } else {
            $response = Http::post($url, json_encode($params), $headers, array(CURLOPT_FAILONERROR => false));
        }
        if (!empty($response['curlError']))
        {
            CrmResponse::replace(array(
                'success' => false,
                'errors' => array(
                    'curlError' => $response['errorMessage'],
                ),
            ));
            return false;
        }

        return $this->prepareResponse($response);
    }

    private function prepareResponse($response)
    {
        $this->response = json_decode($response, true);
        
        if(!empty($this->response) && !empty($this->response['payment_complete_url'])){
            CrmResponse::update(array(
                'success'  => true,
                'redirect' => $this->response['payment_complete_url'],
                'ref_trans_id' => $this->response['reference_transaction_id']
            ));
            return false;
        }
        
        if($this->currentSlug == 'users') {
            if (empty($this->response) || empty($this->response['id']))
            {
                $formattedErrorArray = $this->prepareErrorMsg($response);
                CrmResponse::replace(array(
                    'success' => false,
                    'errors' => array(
                        'crmError' => !empty($formattedErrorArray) ? $formattedErrorArray : $response,
                    ),
                ));
                return false;
            }
        } else {
            if (CrmPayload::get('cardType') == 'offline' && !empty($this->response) && !empty($this->response['number']))  {
                return true;
            }
            else if (empty($this->response) || empty($this->response['number']) || !preg_match('~\b(confirmed|open|archived)\b~i', $this->response['status']))
            {

                if(!empty($this->response['transactions'][0]['payment_details']['card_token']))
                {
                    CrmResponse::replace(array(
                        'success' => false,
                        'errors' => array(
                            'crmError' => $this->response['payment_details'],
                        ),
                        'error_code' => $this->response['payment_response_code'],
                        'card_token' => $this->response['transactions'][0]['payment_details']['card_token']
                    ));
                    return false;
                }

                $formattedErrorArray = $this->prepareErrorMsg($response);
                CrmResponse::replace(array(
                    'success' => false,
                    'errors' => array(
                        'crmError' => !empty($formattedErrorArray) ? $formattedErrorArray : $response,
                    ),
                ));
                return false;
            }
        }

        return true;
    }

    private function prepareErrorMsg($response){
        $errorMessageArray = json_decode($response, true); 
        $formattedErrorArray = array();
        foreach($errorMessageArray as $key => $value) {
            if(is_array($value))
            {
                $this->iterateArray($value, $formattedErrorArray);
            }
            else{

                if(count($errorMessageArray) == 2 && preg_match("/payment/", $key))
                {

                    $formattedErrorArray = array($errorMessageArray[array_keys($errorMessageArray)[0]]." Error Code: ". $errorMessageArray[array_keys($errorMessageArray)[1]]);

                }
                else if(count($errorMessageArray) == 3 && preg_match("/payment/", $key))
                {
                    $formattedErrorArray = array($errorMessageArray[array_keys($errorMessageArray)[0]]);
                    break;
                }
                else{
                    $formattedErrorArray = $errorMessageArray;
                }

            }
        }

        return $formattedErrorArray = implode("<br>", $formattedErrorArray);

    }
    
    private function iterateArray($array, &$formattedErrorArray)
    {
        $unsetKeysArray = array('attribute', 'key');
        foreach($array as $key => $value) {
            if(is_array($value)) {
                $this->iterateArray($value, $formattedErrorArray);
            } else {
                if(!in_array((string) $key, $unsetKeysArray)) {
                    array_push($formattedErrorArray, $value);
                }                
            }
        }
    }

    private function getUrl($slug)
    {
        $url = preg_replace("(^https?://)", "", sprintf('%s/%s/%s/', rtrim($this->endpoint, '/'), 'api/admin', $slug));
        $url = 'https://'. $url;
        return $url;
    }
    
    private function prepareBasicDetails($isProspect = false)
    {
        $basicDetails = array(
            'email' => $this->accessor->getValue($this->params, '[email]'),
            'first_name' => $this->accessor->getValue($this->params, '[firstName]'),
            'last_name' => $this->accessor->getValue($this->params, '[lastName]'),
            'ip' => Request::getClientIp(),
            'phone_number' => !empty($this->accessor->getValue($this->params, '[phone]')) ? $this->params['phone_prefix'].$this->accessor->getValue($this->params, '[phone]') : '',
            'language' => $this->accessor->getValue($this->params, '[language]')
        );

        if(CrmPayload::get('meta.isTwentyNineNextReprocessedOrder')) {
            unset($basicDetails['phone_number']);
        }

        if($isProspect) {
            unset($this->params['language']);
            $this->params = array_replace($this->params, array_filter($basicDetails));
        } else {
            $this->params['user'] = array_filter($basicDetails);
            unset($this->params['email']);
        }
        
    }

    private function prepareShippingDetails($isProspect = false)
    {
        $shippingDetails = array(
            'line1'         => $this->accessor->getValue($this->params, '[shippingAddress1]'),
            'line2'         => $this->accessor->getValue($this->params, '[shippingAddress2]'),
            'postcode'      => $this->accessor->getValue($this->params, '[shippingZip]'),
            'line4'         => $this->accessor->getValue($this->params, '[shippingCity]'),
            'state'         => $this->accessor->getValue($this->params, '[shippingState]'),
            'country'       => $this->accessor->getValue($this->params, '[shippingCountry]'),
            'first_name'    => $this->accessor->getValue($this->params, '[firstName]'),
            'last_name'     => $this->accessor->getValue($this->params, '[lastName]'),
            'phone_number'  => !empty($this->accessor->getValue($this->params, '[phone]')) ? $this->params['phone_prefix'].$this->accessor->getValue($this->params, '[phone]') : ''
        );

        unset(
                $this->params['phone'], $this->params['shippingAddress1'], $this->params['shippingAddress2'], $this->params['shippingZip'], $this->params['shippingCity'], $this->params['shippingState'], $this->params['shippingCountry'], $this->params['ipAddress'], $this->params['campaignId']
        );
        if($isProspect) {
           $this->params['addresses'][0] = array_filter($shippingDetails); 
        } else {
            $this->params['shipping_address'] = array_filter($shippingDetails);
        }        
    }

    private function prepareBillingDetails()
    {
        if (strtolower($this->params['billingSameAsShipping']) !== 'yes')
        {
            $this->params['billing_same_as_shipping_address'] = false;
            $this->params['billing_address'] = array(
                'first_name' => $this->accessor->getValue($this->params, '[billingFirstName]'),
                'last_name' => $this->accessor->getValue($this->params, '[billingLastName]'),
                'line1' => $this->accessor->getValue($this->params, '[billingAddress1]'),
                'line2' => $this->accessor->getValue($this->params, '[billingAddress2]'),
                'postcode' => $this->accessor->getValue($this->params, '[billingZip]'),
                'line4' => $this->accessor->getValue($this->params, '[billingCity]'),
                'state' => $this->accessor->getValue($this->params, '[billingState]'),
                'country' => $this->accessor->getValue($this->params, '[billingCountry]')
            );
            $this->params['billing_address'] = array_filter($this->params['billing_address']);
        }
        else
        {
            $this->params['billing_same_as_shipping_address'] = true;
        }

        unset(
                $this->params['billingFirstName'], $this->params['billingLastName'], $this->params['billingAddress1'], $this->params['billingAddress2'], $this->params['billingCity'], $this->params['billingState'], $this->params['billingZip'], $this->params['billingCountry']
        );
    }

    private function prepareProductDetails($isSubscription = false)
    {
        if(!empty($isSubscription)){
            $this->params['products'] = $isSubscription['products'];
        }

        $productGroup = array();
        foreach ($this->params['products'] as $key => $product)
        {
            if (empty($product['productId']))
            {
                continue;
            }
            $products = array(
                    'sku' => $this->accessor->getValue($product, '[productId]'),
                    'quantity' => (int) $this->accessor->getValue($product, '[productQuantity]'),
                );

            if(!empty(CrmPayload::get('currency'))) {
                $products['currency'] = CrmPayload::get('currency');
            }

            if(
                (CrmPayload::get('meta.stepId') > 1 && CrmPayload::get('meta.crmMethod') != 'newOrderWithCardToken') || 
                (empty($this->params['offers']) && empty($this->params['couponCode']))
            ) {
                if(!empty($product['productPrice']))
                    $products['price'] = $product['productPrice'];
            }
            
            if (!empty($product['subscription'])) {
                $products['subscription']['interval'] = $product['subscription']['interval'];
                $products['subscription']['interval_count'] = (int)$product['subscription']['interval_count'];
                if (!empty($product['subscription']['price'])) {
                    $products['subscription']['price'] = $product['subscription']['price'];
                }
            }

            if($key > 0) {
                if(empty(CrmPayload::get('ignore_isupsell'))) {
                    $products['is_upsell'] = true;
                }
                if(!empty($product['is_upsell_product'])) {
                    $products['is_upsell'] = true;
                }
                if(empty($product['ignore_isupsell_price'])) {
                    $products['price'] = $product['productPrice'];
                }  
            }
            
            if (!empty(CrmPayload::get('meta.total_price'))  && !empty(CrmPayload::get('meta.product_total_price_id'))) {

                if($product['productId'] == CrmPayload::get('meta.product_total_price_id')){

                    $products['price'] = CrmPayload::get('meta.total_price');

                }

            }
            
            if(!empty(CrmPayload::get('stockrecord_id'))) {
                $products['stockrecord_id'] = $this->accessor->getValue($product, '[productId]');
                unset($products['sku'], $products['currency']);
            }
            
            if (!empty(CrmPayload::get('meta.enable_product_id'))) {
                $products['product_id'] = $this->accessor->getValue($product, '[productId]');
                if (!empty(CrmPayload::get('meta.currency'))) {
                    $products['currency'] = CrmPayload::get('meta.currency');
                } elseif (empty(CrmPayload::get('currency'))) {
                    $products['currency'] = 'USD';
                }
                unset($products['sku']);
              
            } 
            
            array_push($productGroup, $products);
        }

        if((CrmPayload::get('meta.stepId') == 1 || !empty(CrmPayload::get('meta.allow_shipping_id'))) && !empty($this->params['products'][0]['shippingId'])) {
            $this->params['shipping_code'] = $this->params['products'][0]['shippingId'];
        }

        if((!empty(CrmPayload::get('meta.allow_shipping_price'))) && !empty($this->params['products'][0]['shippingPrice'])) {
            $this->params['shipping_price'] = $this->params['products'][0]['shippingPrice'];
        }

        if(!empty(CrmPayload::get('meta.shipping_price'))){
            $this->params['shipping_price'] = CrmPayload::get('meta.shipping_price');
        }
        
        unset($this->params['products'], $this->params['currency']);
        $this->params['lines'] = $productGroup;    
    }

    private function prepareCardDetails()
    {
        if($this->params['cardType'] == 'paypal') {
            $paymentInfo['payment_return_url'] = $this->params['payment_return_url'];
            $this->params['payment_method'] = 'paypal';
        } else if($this->params['cardType'] == 'klarna') {
            $paymentInfo['payment_return_url'] = $this->params['payment_return_url'];
            $this->params['payment_method'] = 'klarna';
        } else if($this->params['cardType'] == 'COD') {
            $this->params['payment_method'] = 'cash_on_delivery';
        } else if($this->params['cardType'] == 'offline') {
            $this->params['payment_method'] = 'bank_transfer';
        } 
        else if($this->params['cardType'] == 'ideal') {
            $paymentInfo['payment_return_url'] = $this->params['payment_return_url'];
            $this->params['payment_method'] = 'ideal';
        } 
        else if($this->params['cardType'] == 'bancontact') {
            $paymentInfo['payment_return_url'] = $this->params['payment_return_url'];
            $this->params['payment_method'] = 'bancontact';
        }
        else if($this->params['cardType'] == 'giropay') {
            $paymentInfo['payment_return_url'] = $this->params['payment_return_url'];
            $this->params['payment_method'] = 'giropay';
        }    
        else if($this->params['cardType'] == 'external') {
            $this->params['payment_method'] = 'card_token';
            $paymentInfo['card_token'] = $this->params['card_token'];
            if (!empty($this->params['payment_return_url']))
            {
                $paymentInfo['payment_return_url'] = $this->params['payment_return_url'];
            }
            if (!empty($this->params['forceGatewayId']))
            {
                $paymentInfo['payment_gateway'] = $this->params['forceGatewayId'];
            }
            if (!empty($this->params['payment_gateway_group']))
            {
                $paymentInfo['payment_gateway_group'] = $this->params['payment_gateway_group'];
            }
        }
        else if($this->params['cardType'] == 'apple_pay') {
            
            $paymentInfo['payment_return_url'] = $this->params['payment_details']['payment_return_url'];
            $this->params['payment_method'] = 'apple_pay';
            unset($this->params['user']);
            unset($this->params['shipping_address']);
            unset($this->params['billing_same_as_shipping_address']);
            unset($this->params['shipping_code']);
        } 
        else if($this->params['cardType'] == 'google_pay') {
            
            $paymentInfo['payment_return_url'] = $this->params['payment_return_url'];
            $this->params['payment_method'] = 'google_pay';
            unset($this->params['user']);
            unset($this->params['shipping_address']);
            unset($this->params['billing_same_as_shipping_address']);
            unset($this->params['shipping_code']);
        }
        else {

            if(
                !empty($this->params['isCascaseFlow']) && 
                !empty($this->params['card_token']) &&
                !empty($this->params['forceCRMCascadeGatewayId'])
            ){
                $this->params['payment_method'] = 'card_token';
                $paymentInfo['payment_gateway'] = $this->params['forceCRMCascadeGatewayId'];
                $paymentInfo['card_token'] = $this->params['card_token'];

                unset($this->params['card_token'], $this->params['isCascaseFlow'], $this->params['forceCRMCascadeGatewayId']);
            }
            else{
                $paymentInfo = array(
                    'card_name' => $this->params['firstName'] . ' ' . $this->params['lastName'],
                    'card_number' => $this->accessor->getValue($this->params, '[cardNumber]'),
                    'card_expiry_month' => $this->accessor->getValue($this->params, '[cardExpiryMonth]'),
                    'card_expiry_year' => !empty($this->params['cardExpiryYear']) ? $this->accessor->getValue($this->params, '[cardExpiryYear]') : '',
                    'card_cvv' => $this->accessor->getValue($this->params, '[cvv]'),
                    'save_card' => true
                );

                if (!empty($this->params['forceGatewayId']))
                {
                    $paymentInfo['payment_gateway'] = $this->params['forceGatewayId'];
                }

                if (!empty($this->params['payment_return_url']))
                {
                    $paymentInfo['payment_return_url'] = $this->params['payment_return_url'];
                }

                if (!empty($this->params['payment_gateway_group']))
                {
                    $paymentInfo['payment_gateway_group'] = $this->params['payment_gateway_group'];
                }

                $this->params['payment_method'] = 'bankcard';
            }
        }  
        
        unset(
                    $this->params['cardExpiryMonth'], $this->params['cardExpiryYear'], $this->params['cvv'], $this->params['productDetails'], $this->params['payment_return_url'], $this->params['payment_gateway_group']
            );
        
        $this->params['payment_details'] = $paymentInfo;
    }

    private function prepareAffiliates()
    {
        $affiliates = $this->params['affiliates'];

        $affiliateParams = array_filter(array(
            'affiliate' => $this->accessor->getValue($affiliates, '[affId]'),
            'subaffiliate1' => $this->accessor->getValue($affiliates, '[c1]'),
            'subaffiliate2' => $this->accessor->getValue($affiliates, '[c2]'),
            'subaffiliate3' => $this->accessor->getValue($affiliates, '[c3]'),
            'subaffiliate4' => $this->accessor->getValue($affiliates, '[c4]'),
            'subaffiliate5' => $this->accessor->getValue($affiliates, '[c5]'),
            "utm_source"    => $this->accessor->getValue($affiliates, '[utm_source]'),
            "utm_medium"    => $this->accessor->getValue($affiliates, '[utm_medium]'),
            "utm_campaign"  => $this->accessor->getValue($affiliates, '[utm_campaign]'),
            "utm_term"      => $this->accessor->getValue($affiliates, '[utm_term]'),
            "utm_content"   => $this->accessor->getValue($affiliates, '[utm_content]'),
            "funnel"        => $this->accessor->getValue($affiliates, '[funnel]'),
            "gclid"         => $this->accessor->getValue($affiliates, '[gclid]'),
            "evclid"        => $this->accessor->getValue($affiliates, '[evclid]'),
            "pid"           => $this->accessor->getValue($affiliates, '[pid]'),
            "vid"           => $this->accessor->getValue($affiliates, '[vid]')
        ));

        unset($this->params['affiliates']);

        $this->params['attribution'] = $affiliateParams;
        if(!empty($affiliateParams['evclid']))
        {
            $this->params['attribution']['metadata']['everflow_transaction_id'] = $affiliateParams['evclid'];
            unset($this->params['attribution']['evclid']);
        }
        if(!empty($affiliateParams['pid']))
        {
            $this->params['attribution']['metadata']['pid'] = $affiliateParams['pid'];
            unset($this->params['attribution']['pid']);
        }
        if(!empty($affiliateParams['vid']))
        {
            $this->params['attribution']['metadata']['vid'] = $affiliateParams['vid'];
            unset($this->params['attribution']['vid']);
        }
        if(!empty($this->params['domain']))
        {
            $this->params['attribution']['metadata']['domain'] = $this->params['domain'];
            //$this->params['metadata']['domain'] = $this->params['domain'];
            unset($this->params['domain']);
        }
        if(!empty($this->params['device']))
        {
            $this->params['attribution']['metadata']['device'] = $this->params['device'];
            //$this->params['metadata']['device'] = $this->params['device'];
            unset($this->params['device']);
        }

        if(!empty($this->params['attribution_custom_meta']) && is_array($this->params['attribution_custom_meta']))
        {
            foreach ($this->params['attribution_custom_meta'] as $key => $value) {
               $this->params['attribution']['metadata'][$key] = $value;
            }
            unset($this->params['attribution_custom_meta']);
        }
    }

    private function cleanParamsBeforePost()
    {
        unset($this->params['prospectId'], $this->params['firstName'], $this->params['lastName'], $this->params['userIsAt'], $this->params['billingSameAsShipping'], $this->params['billingFirstName'], $this->params['billingLastName'], $this->params['billingAddress1'], $this->params['billingZip'], $this->params['billingCity'], $this->params['billingState'], $this->params['billingCountry'], $this->params['cardType'], $this->params['cardNumber'], $this->params['SiteID'], $this->params['FirstName'], $this->params['LastName'], $this->params['Email'], $this->params['Phone'], $this->params['Address1'], $this->params['Address2'], $this->params['ZipCode'], $this->params['City'], $this->params['State'], $this->params['CountryISO'], $this->params['campaignId'], $this->params['ipAddress'], $this->params['downsellCustomerId'], $this->params['x_transaction_id'], $this->params['ignore_isupsell']);
    }

    private function GetCustomerAddress()
    {
        return $shippingDetails = array(
            'FirstName' => $this->accessor->getValue($this->params, '[FirstName]'),
            'LastName' => $this->accessor->getValue($this->params, '[LastName]'),
            'Email' => $this->accessor->getValue($this->params, '[Email]'),
            'Phone' => $this->accessor->getValue($this->params, '[Phone]'),
            'Address1' => $this->accessor->getValue($this->params, '[Address1]'),
            'Address2' => $this->accessor->getValue($this->params, '[Address2]'),
            'ZipCode' => $this->accessor->getValue($this->params, '[ZipCode]'),
            'City' => $this->accessor->getValue($this->params, '[City]'),
            'State' => $this->accessor->getValue($this->params, '[State]'),
            'CountryISO' => $this->accessor->getValue($this->params, '[CountryISO]')
        );
    }

    public function preAuthorization()
    {
        CrmResponse::replace(array(
            'success'    => true,
            'orderId'    => strtoupper(uniqid()),
            'customerId' => strtoupper(uniqid()),
        ));
    }
    
    public function createPaypalPayment()
    {

        $this->beforeAnyCrmClassMethodCall();

        $this->params = array_replace($this->params, CrmPayload::all());
        $headers = array(
            'Content-Type' => 'application/json', 
            'Authorization' => 'ApiKey ' . $this->params['CRMAPIKey']
        );
        unset($this->params['CRMAPIKey']);
        $url = $this->getUrl($this->methodSlugMapping['createPaypalPayment']);
        $response = Http::post($url, json_encode($this->params), $headers);
        $response = json_decode($response, true);
        if (!empty($response['curlError']))
        {
            CrmResponse::replace(array(
                'success' => false,
                'errors' => array(
                    'curlError' => $response['errorMessage'],
                ),
            ));
            return false;
        }
        
        if(!empty($response['PaymentID']))
        {
            CrmResponse::replace(array(
                'success'       => true,
                'CrmOrderID'    => $response['CrmOrderID'],
                'PaymentID'     => $response['PaymentID'],
            ));
        }
        else{
            CrmResponse::replace(array(
                'success'       => false,
                'crmError'      => $response['ErrorMessage']
            ));
        }
    }
    
    public function executePaypalPayment()
    {
        $this->beforeAnyCrmClassMethodCall();

        $this->params = array_replace($this->params, CrmPayload::all());
        $headers = array(
            'Content-Type' => 'application/json', 
            'Authorization' => 'ApiKey ' . $this->params['CRMAPIKey']
        );
        unset($this->params['CRMAPIKey']);
        $url = $this->getUrl($this->methodSlugMapping['executePaypalPayment']);
        $response = Http::post($url, json_encode($this->params), $headers);

        if ($this->prepareResponse($response) === false)
        {
            return false;
        }

        CrmResponse::replace(array(
            'success' => true,
            'orderId' => $this->response['Transaction']['OrderInfo']['TransactionID'],
            'crmOrderId' => $this->response['Transaction']['OrderInfo']['OrderID'],
            'customerId' => $this->response['Transaction']['OrderInfo']['CustomerID'],
            'transactionInfo' => $this->response['Transaction']
        ));

    }

    public static function isValidCredential($credential)
    {
        if (is_array($credential) && !empty($credential))
        {
            $url = preg_replace("(^https?://)", "", sprintf('%s/%s/%s/', rtrim($credential['endpoint'], '/'), 'api/admin', 'products'));
            $url = 'https://'. $url;
            $authorization = 'Token ';
            
            if(!empty($credential['twentyninenext_token_type'] && $credential['twentyninenext_token_type'] == 'oauth_token')){
                $authorization = 'Bearer ';
            }

            try
            {
                $headers = array('Content-Type' => 'application/json', 'Authorization' => $authorization . $credential['username']);
                $response = Http::get($url, $headers, array(CURLOPT_FAILONERROR => false));
                
                $responseArray = json_decode($response, true);
                
                if (!empty($responseArray) && is_array($responseArray) && !empty($responseArray['results']))
                {
                    return true;
                }

                return false;
            }
            catch (Exception $ex)
            {
                return false;
            }
        }
    }

    public function orderView()
    {
        $this->beforeAnyCrmClassMethodCall();

        $this->params['method'] = $this->memberShipMethods['orderView'];

        $orderIds       = CrmPayload::get('orderIds');
        $uniqueOrderIds = array_values(array_unique($orderIds));
        $result         = $data         = array();

        $this->params['order_number'] = implode(',', $uniqueOrderIds);
        
        $url = preg_replace("(^https?://)", "", sprintf('%s/%s/%s/', rtrim($this->endpoint, '/'), 'api/admin', $this->params['method']));
        $url = 'https://'. $url .'?order_number='. $this->params['order_number'];
        
        $headers = array('Content-Type' => 'application/json', 'Authorization' => $this->token . $this->username);
        unset($this->params['method']);
        $response = Http::get($url, $headers, array(CURLOPT_FAILONERROR => false));
        
        $responseArray = json_decode($response, true);
        
        if (empty($responseArray['results'])) {
            $data[$uniqueOrderIds[0]] = $response;
        } else {
            $data = $responseArray['results'];
        }

        foreach ($orderIds as $key => $orderId) {
            if (!empty($data[$key]['number']) &&  $data[$key]['number'] == $orderId && is_array($data[$key])) {
                $result[$orderId] = $data[$key];
            } else {
                $result[$key] = 'Not found!';
            }
        }

        CrmResponse::replace(array(
            'success' => true,
            'result'  => $result,
        ));
    }

}
