<?php

declare(strict_types=1);

namespace Lazer\Test\Classes\Helpers;
use Lazer\Classes\Helpers\Validate;
use Lazer\Test\VfsHelper\Config as TestHelper;

/**
 * Generated by PHPUnit_SkeletonGenerator on 2015-04-03 at 16:22:49.
 */
class ValidateTest extends \PHPUnit\Framework\TestCase {

    use TestHelper;

    /**
     * @var Validate
     */
    protected $object;

    /**
     * Sets up the fixture, for example, opens a network connection.
     * This method is called before a test is executed.
     */
    protected function setUp()
    {
        $this->setUpFilesystem();
        $this->object = new Validate;
    }

    /**
     * Tears down the fixture, for example, closes a network connection.
     * This method is called after a test is executed.
     */
    protected function tearDown()
    {
        
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::table
     * @todo   Implement testTable().
     */
    public function testTable()
    {
        $object = $this->object->table('users');
        $this->assertInstanceOf('Lazer\Classes\Helpers\Validate', $object);
        return $object;
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::isNumeric
     */
    public function testIsNumeric()
    {
        $this->assertTrue($this->object->isNumeric('integer'));
        $this->assertTrue($this->object->isNumeric('double'));
        $this->assertFalse($this->object->isNumeric('boolean'));
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::types
     */
    public function testTypesMatches()
    {
        $this->assertTrue($this->object->types(['integer', 'double', 'string', 'boolean']));
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::types
     * @expectedException \Lazer\Classes\LazerException
     */
    public function testTypesNotMatches()
    {
        $this->object->types(['integer', 'float']);
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::filter
     */
    public function testFilterIndexedArrayFromIdKey()
    {
        $arrayOne = ['id', 'column1', 'column2'];
        $this->assertArrayNotHasKey('id', $this->object->filter($arrayOne));
        $arrayTwo = ['id' => 1, 'column1' => 'value1', 'column2' => 'value2'];
        $this->assertArrayNotHasKey('id', $this->object->filter($arrayTwo));
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::arrToLower
     */
    public function testArrToLower()
    {
        $array = $this->object->arrToLower(['String1', 'StRIng2']);
        $this->assertSame(['string1', 'string2'], $array);
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::fields
     * @depends testTable
     */
    public function testFieldsExistsInTable($object)
    {
        $array = ['name', 'email'];
        $this->assertTrue($object->fields($array));
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::fields
     * @depends testTable
     * @expectedException \Lazer\Classes\LazerException
     */
    public function testFieldsNotExistsInTable($object)
    {
        $array = ['name', 'surname', 'city'];
        $object->fields($array);
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::field
     * @depends testTable
     */
    public function testFieldExistsInTable($object)
    {
        $field = 'name';
        $this->assertTrue($object->field($field));
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::field
     * @depends testTable
     * @expectedException \Lazer\Classes\LazerException
     * @expectedExceptionMessage Field surname does not exists in table "users"
     */
    public function testFieldNotExistsInTable($object)
    {
        $field = 'surname';
        $object->field($field);
    }


    /**
     * @covers \Lazer\Classes\Helpers\Validate::exists
     * @depends testTable
     */
    public function testTableExists($object)
    {
        $this->assertTrue($object->exists());
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::exists
     * @expectedException \Lazer\Classes\LazerException
     */
    public function testTableNotExists()
    {
        $object = $this->object->table('virtual');
        $object->exists();
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::type
     * @depends testTable
     */
    public function testTypeIsValid($object)
    {
        $this->assertTrue($object->type('name', 'Henry'));
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::type
     * @depends testTable
     * @expectedException \Lazer\Classes\LazerException
     */
    public function testTypeIsNotValid($object)
    {
        $object->type('name', 123);
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::relation
     */
    public function testRelationExists()
    {
        $this->assertTrue($this->object->relation('users', 'news'));
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::relation
     * @expectedException \Lazer\Classes\LazerException
     */
    public function testRelationNotExists()
    {
        $this->object->relation('users', 'tags');
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::relationType
     */
    public function testRelationTypeExists()
    {
        $this->assertTrue($this->object->relationType('belongsTo'));
    }

    /**
     * @covers \Lazer\Classes\Helpers\Validate::relationType
     * @expectedException \Lazer\Classes\LazerException
     */
    public function testRelationTypeNotExists()
    {
        $this->object->relationType('someRelation');
    }

}
